@@ -7,6 +7,8 @@ Components.utils.import("resource://calendar/modules/calUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource://gre/modules/Preferences.jsm");
 
+/* exported modifyEventWithDialog, undo, redo, setContextPartstat */
+
 /**
  * Takes a job and makes sure the dispose function on it is called. If there is
  * no dispose function or the job is null, ignore it.
@@ -80,19 +82,18 @@ function setDefaultItemValues(aItem, aCalendar=null, aStartDate=null, aEndDate=n
             }
         } else {
             aItem.endDate = aItem.startDate.clone();
-            if (!aForceAllday) {
+            if (aForceAllday) {
+                // All day events need to go to the beginning of the next day.
+                aItem.endDate.day++;
+            } else {
                 // If the event is not all day, then add the default event
                 // length.
                 aItem.endDate.minute += Preferences.get("calendar.event.defaultlength", 60);
-            } else {
-                // All day events need to go to the beginning of the next day.
-                aItem.endDate.day++;
             }
         }
 
         // Free/busy status is only valid for events, must not be set for tasks.
         aItem.setProperty("TRANSP", cal.getEventDefaultTransparency(aForceAllday));
-
     } else if (cal.isToDo(aItem)) {
         let now = cal.now();
         let initDate = initialDate ? initialDate.clone() : now;
@@ -226,16 +227,14 @@ function setDefaultItemValues(aItem, aCalendar=null, aStartDate=null, aEndDate=n
  *                                   allday event.
  */
 function createEventWithDialog(calendar, startDate, endDate, summary, event, aForceAllday) {
-    const kDefaultTimezone = calendarDefaultTimezone();
-
-    var onNewEvent = function(item, calendar, originalItem, listener) {
+    let onNewEvent = function(item, opcalendar, originalItem, listener) {
         if (item.id) {
             // If the item already has an id, then this is the result of
             // saving the item without closing, and then saving again.
-            doTransaction('modify', item, calendar, originalItem, listener);
+            doTransaction("modify", item, opcalendar, originalItem, listener);
         } else {
             // Otherwise, this is an addition
-            doTransaction('add', item, calendar, null, listener);
+            doTransaction("add", item, opcalendar, null, listener);
         }
     };
 
@@ -270,7 +269,7 @@ function createEventWithDialog(calendar, startDate, endDate, summary, event, aFo
             event.title = summary;
         }
     }
-    openEventDialog(event, event.calendar, "new", onNewEvent, null);
+    openEventDialog(event, event.calendar, "new", onNewEvent);
 }
 
 /**
@@ -283,18 +282,16 @@ function createEventWithDialog(calendar, startDate, endDate, summary, event, aFo
  * @param initialDate   (optional) The initial date for new task datepickers
  */
 function createTodoWithDialog(calendar, dueDate, summary, todo, initialDate) {
-    const kDefaultTimezone = calendarDefaultTimezone();
-
-    var onNewItem = function(item, calendar, originalItem, listener) {
+    let onNewItem = function(item, opcalendar, originalItem, listener) {
         if (item.id) {
             // If the item already has an id, then this is the result of
             // saving the item without closing, and then saving again.
-            doTransaction('modify', item, calendar, originalItem, listener);
+            doTransaction("modify", item, opcalendar, originalItem, listener);
         } else {
             // Otherwise, this is an addition
-            doTransaction('add', item, calendar, null, listener);
+            doTransaction("add", item, opcalendar, null, listener);
         }
-    }
+    };
 
     if (todo) {
         // If the todo should be created from a template, then make sure to
@@ -320,8 +317,6 @@ function createTodoWithDialog(calendar, dueDate, summary, todo, initialDate) {
     openEventDialog(todo, calendar, "new", onNewItem, null, initialDate);
 }
 
-
-
 /**
  * Modifies the passed event in the event dialog.
  *
@@ -331,8 +326,21 @@ function createTodoWithDialog(calendar, dueDate, summary, todo, initialDate) {
  * @param aPromptOccurrence     If the user should be prompted to select if the
  *                                parent item or occurrence should be modified.
  * @param initialDate           (optional) The initial date for new task datepickers
+ * @param aCounterProposal      (optional) An object representing the counterproposal
+ *        {
+ *            {JsObject} result: {
+ *                type: {String} "OK"|"OUTDATED"|"NOTLATESTUPDATE"|"ERROR"|"NODIFF"
+ *                descr: {String} a technical description of the problem if type is ERROR or NODIFF,
+ *                                otherwise an empty string 
+ *            },
+ *            (empty if result.type = "ERROR"|"NODIFF"){Array} differences: [{
+ *                property: {String} a property that is subject to the proposal
+ *                proposed: {String} the proposed value
+ *                original: {String} the original value
+ *            }]
+ *        }
  */
-function modifyEventWithDialog(aItem, job, aPromptOccurrence, initialDate) {
+function modifyEventWithDialog(aItem, job=null, aPromptOccurrence, initialDate=null, aCounterProposal) {
     let dlg = cal.findItemWindow(aItem);
     if (dlg) {
         dlg.focus();
@@ -341,17 +349,18 @@ function modifyEventWithDialog(aItem, job, aPromptOccurrence, initialDate) {
     }
 
     let onModifyItem = function(item, calendar, originalItem, listener) {
-        doTransaction('modify', item, calendar, originalItem, listener);
+        doTransaction("modify", item, calendar, originalItem, listener);
     };
 
     let item = aItem;
-    let futureItem, response;
+    let response;
     if (aPromptOccurrence !== false) {
-        [item, futureItem, response] = promptOccurrenceModification(aItem, true, "edit");
+        [item, , response] = promptOccurrenceModification(aItem, true, "edit");
     }
 
     if (item && (response || response === undefined)) {
-        openEventDialog(item, item.calendar, "modify", onModifyItem, job, initialDate);
+        openEventDialog(item, item.calendar, "modify", onModifyItem, job, initialDate,
+                        aCounterProposal);
     } else {
         disposeJob(job);
     }
@@ -366,8 +375,10 @@ function modifyEventWithDialog(aItem, job, aPromptOccurrence, initialDate) {
  * @param callback          The callback to call when the dialog has completed.
  * @param job               (optional) The job object for the modification.
  * @param initialDate       (optional) The initial date for new task datepickers
+ * @param counterProposal   (optional) An object representing the counterproposal - see
+ *                                     description for modifyEventWithDialog()
  */
-function openEventDialog(calendarItem, calendar, mode, callback, job, initialDate) {
+function openEventDialog(calendarItem, calendar, mode, callback, job=null, initialDate=null, counterProposal) {
     let dlg = cal.findItemWindow(calendarItem);
     if (dlg) {
         dlg.focus();
@@ -378,16 +389,16 @@ function openEventDialog(calendarItem, calendar, mode, callback, job, initialDat
     // Set up some defaults
     mode = mode || "new";
     calendar = calendar || getSelectedCalendar();
-    var calendars = getCalendarManager().getCalendars({});
+    let calendars = getCalendarManager().getCalendars({});
     calendars = calendars.filter(isCalendarWritable);
 
-    var isItemSupported;
+    let isItemSupported;
     if (isToDo(calendarItem)) {
-        isItemSupported = function isTodoSupported(aCalendar) {
+        isItemSupported = function(aCalendar) {
             return (aCalendar.getProperty("capabilities.tasks.supported") !== false);
         };
     } else if (isEvent(calendarItem)) {
-        isItemSupported = function isEventSupported(aCalendar) {
+        isItemSupported = function(aCalendar) {
             return (aCalendar.getProperty("capabilities.events.supported") !== false);
         };
     }
@@ -405,24 +416,25 @@ function openEventDialog(calendarItem, calendar, mode, callback, job, initialDat
              * check that the user can remove items from that calendar and
              * add items to the current one.
              */
-            return (((calendarItem.calendar != aCalendar)
-                     && userCanDeleteItemsFromCalendar(calendarItem.calendar)
-                     && userCanAddItemsToCalendar(aCalendar))
-                    || ((calendarItem.calendar == aCalendar)
-                        && userCanModifyItem(calendarItem)));
+            let isSameCalendar = calendarItem.calendar == aCalendar;
+            let canModify = userCanModifyItem(calendarItem);
+            let canMoveItems = userCanDeleteItemsFromCalendar(calendarItem.calendar) &&
+                               userCanAddItemsToCalendar(aCalendar);
+
+            return isSameCalendar ? canModify : canMoveItems;
         });
     }
 
-    if (mode == "new"
-        && (!isCalendarWritable(calendar)
-            || !userCanAddItemsToCalendar(calendar)
-            || !isItemSupported(calendar))) {
+    if (mode == "new" &&
+        (!isCalendarWritable(calendar) ||
+         !userCanAddItemsToCalendar(calendar) ||
+         !isItemSupported(calendar))) {
         if (calendars.length < 1) {
             // There are no writable calendars or no calendar supports the given
             // item. Don't show the dialog.
             disposeJob(job);
             return;
-        } else  {
+        } else {
             // Pick the first calendar that supports the item and is writable
             calendar = calendars[0];
             if (calendarItem) {
@@ -435,20 +447,23 @@ function openEventDialog(calendarItem, calendar, mode, callback, job, initialDat
     }
 
     // Setup the window arguments
-    var args = new Object();
+    let args = {};
     args.calendarEvent = calendarItem;
     args.calendar = calendar;
     args.mode = mode;
     args.onOk = callback;
     args.job = job;
-    args.initialStartDateValue = (initialDate || getDefaultStartDate());
+    args.initialStartDateValue = initialDate || getDefaultStartDate();
+    args.counterProposal = counterProposal;
+    args.inTab = Preferences.get("calendar.item.editInTab", false);
+    args.useNewItemUI = Preferences.get("calendar.item.useNewItemUI", false);
 
     // this will be called if file->new has been selected from within the dialog
-    args.onNewEvent = function(calendar) {
-        createEventWithDialog(calendar, null, null);
+    args.onNewEvent = function(opcalendar) {
+        createEventWithDialog(opcalendar, null, null);
     };
-    args.onNewTodo = function(calendar) {
-        createTodoWithDialog(calendar);
+    args.onNewTodo = function(opcalendar) {
+        createTodoWithDialog(opcalendar);
     };
 
     // the dialog will reset this to auto when it is done loading.
@@ -462,27 +477,42 @@ function openEventDialog(calendarItem, calendar, mode, callback, job, initialDat
 
     // open the dialog modeless
     let url;
-    if (isCalendarWritable(calendar)
-        && (mode == "new"
-            || (mode == "modify" && !isInvitation && userCanModifyItem((calendarItem))))) {
-        url = "chrome://calendar/content/calendar-event-dialog.xul";
+    let isEditable = mode == "modify" && !isInvitation && userCanModifyItem(calendarItem);
+    if (isCalendarWritable(calendar) && (mode == "new" || isEditable)) {
+        if (args.inTab) {
+            url = args.useNewItemUI ? "chrome://lightning/content/html-item-editing/lightning-item-iframe.html"
+                                    : "chrome://lightning/content/lightning-item-iframe.xul";
+        } else {
+            url = "chrome://calendar/content/calendar-event-dialog.xul";
+        }
     } else {
         url = "chrome://calendar/content/calendar-summary-dialog.xul";
+        args.inTab = false;
     }
 
-    // reminder: event dialog should not be modal (cf bug 122671)
-    var features;
-    // keyword "dependent" should not be used (cf bug 752206)
-    if (Services.appinfo.OS == "WINNT") {
-        features = "chrome,titlebar,resizable";
-    } else if (Services.appinfo.OS == "Darwin") {
-        features = "chrome,titlebar,resizable,minimizable=no";
+    if (args.inTab) {
+        // open in a tab, currently the read-only summary dialog is
+        // never opened in a tab
+        args.url = url;
+        let tabmail = document.getElementById("tabmail");
+        let tabtype = cal.isEvent(args.calendarEvent) ? "calendarEvent" : "calendarTask";
+        tabmail.openTab(tabtype, args);
     } else {
-        // All other targets, mostly Linux flavors using gnome.
-        features = "chrome,titlebar,resizable,minimizable=no,dialog=no";
-    }
+        // open in a window
 
-    openDialog(url, "_blank", features, args);
+        // reminder: event dialog should not be modal (cf bug 122671)
+        let features;
+        // keyword "dependent" should not be used (cf bug 752206)
+        if (Services.appinfo.OS == "WINNT") {
+            features = "chrome,titlebar,resizable";
+        } else if (Services.appinfo.OS == "Darwin") {
+            features = "chrome,titlebar,resizable,minimizable=no";
+        } else {
+            // All other targets, mostly Linux flavors using gnome.
+            features = "chrome,titlebar,resizable,minimizable=no,dialog=no";
+        }
+        openDialog(url, "_blank", features, args);
+    }
 }
 
 /**
@@ -527,9 +557,9 @@ function promptOccurrenceModification(aItem, aNeedsFuture, aAction) {
     const MODIFY_FOLLOWING = 2;
     const MODIFY_PARENT = 3;
 
-    var futureItem = false;
-    var pastItem;
-    var type = CANCEL;
+    let futureItem = false;
+    let pastItem;
+    let type = CANCEL;
 
     // Check if this actually is an instance of a recurring event
     if (aItem == aItem.parentItem) {
@@ -542,7 +572,7 @@ function promptOccurrenceModification(aItem, aNeedsFuture, aAction) {
     } else {
         // Prompt the user. Setting modal blocks the dialog until it is closed. We
         // use rv to pass our return value.
-        var rv = { value: CANCEL, item: aItem, action: aAction};
+        let rv = { value: CANCEL, item: aItem, action: aAction };
         window.openDialog("chrome://calendar/content/calendar-occurrence-prompt.xul",
                           "PromptOccurrenceModification",
                           "centerscreen,chrome,modal,titlebar",
@@ -557,7 +587,6 @@ function promptOccurrenceModification(aItem, aNeedsFuture, aAction) {
         case MODIFY_FOLLOWING:
             // TODO tbd in a different bug
             throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
-            break;
         case MODIFY_OCCURRENCE:
             pastItem = aItem;
             break;
@@ -672,7 +701,7 @@ function updateUndoRedoMenu() {
 function setContextPartstat(value, scope, items) {
     startBatchTransaction();
     try {
-        for each (let oldItem in items) {
+        for (let oldItem of items) {
             // Skip this item if its calendar is read only.
             if (oldItem.calendar.readOnly) {
                 continue;
@@ -704,7 +733,7 @@ function setContextPartstat(value, scope, items) {
                     newItem.addAttendee(newAttendee);
                 }
 
-                doTransaction('modify', newItem, newItem.calendar, oldItem, null);
+                doTransaction("modify", newItem, newItem.calendar, oldItem, null);
             }
         }
     } catch (e) {
