@@ -8,6 +8,7 @@
 Components.utils.import("resource:///modules/jsTreeSelection.js");
 Components.utils.import("resource:///modules/MailUtils.js");
 Components.utils.import("resource://gre/modules/Services.jsm");
+Components.utils.import("resource://gre/modules/AppConstants.jsm");
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource:///modules/MsgHdrSyntheticView.js");
 
@@ -75,7 +76,9 @@ StandaloneFolderDisplayWidget.prototype = {
   onCreatedView:
       function StandaloneMessageDisplayWidget_onCreatedView() {
     this._fakeTreeBox.view = this.view.dbView;
+    // Need to clear out this reference later.
     this._magicTreeSelection.view = this.view.dbView;
+
     // only if we're not dealing with a dummy message (from .eml file /
     //  attachment should we try and hook up the selection object.)  Otherwise
     //  the view will not operate in stand alone message mode.
@@ -224,13 +227,13 @@ StandaloneMessageDisplayWidget.prototype = {
 
     // If the tab hasn't got a title, or we're on Mac, don't display
     // the separator.
-    if (docTitle && !Application.platformIsMac)
+    if (docTitle && (AppConstants.platform != "macosx"))
         docTitle += document.documentElement
                             .getAttribute("titlemenuseparator");
 
     // If we haven't got a title at this stage add the modifier, or if
     // we are on a non-mac platform, add the modifier.
-    if (!docTitle || !Application.platformIsMac)
+    if (!docTitle || (AppConstants.platform != "macosx"))
          docTitle += document.documentElement
                              .getAttribute("titlemodifier");
 
@@ -766,7 +769,9 @@ function HideMenus()
 function OnUnloadMessageWindow()
 {
   if (gFolderDisplay._magicTreeSelection) {
+    // Avoid cycle leaks.
     gFolderDisplay._magicTreeSelection.tree = null;
+    gFolderDisplay._magicTreeSelection.view = null;
     gFolderDisplay._magicTreeSelection = null;
   }
   gFolderDisplay.close();
@@ -885,6 +890,7 @@ var MessageWindowController =
       case "cmd_forwardInline":
       case "cmd_forwardAttachment":
       case "cmd_editAsNew":
+      case "cmd_editDraftMsg":
       case "cmd_getNextNMessages":
       case "cmd_find":
       case "cmd_findAgain":
@@ -967,6 +973,7 @@ var MessageWindowController =
       case "cmd_forwardInline":
       case "cmd_forwardAttachment":
       case "cmd_editAsNew":
+      case "cmd_editDraftMsg":
       case "cmd_print":
       case "cmd_printpreview":
       case "button_print":
@@ -1133,7 +1140,10 @@ var MessageWindowController =
         MsgForwardAsAttachment(null);
         break;
       case "cmd_editAsNew":
-        MsgEditMessageAsNew();
+        MsgEditMessageAsNew(null);
+        break;
+      case "cmd_editDraftMsg":
+        MsgEditDraftMessage(null);
         break;
       case "cmd_moveToFolderAgain":
         var folder = MailUtils.getFolderForURI(
