@@ -298,10 +298,14 @@ MultiMessageSummary.prototype = {
         if (meta.author)
           authorNode.textContent = meta.author;
       }, false, {saneBodySize: true});
-    } catch (e if e.result == Components.results.NS_ERROR_FAILURE) {
-      // Offline messages generate exceptions, which is unfortunate.  When
-      // that's fixed, this code should adapt. XXX
-      snippetNode.textContent = "...";
+    } catch (e) {
+      if (e.result == Components.results.NS_ERROR_FAILURE) {
+        // Offline messages generate exceptions, which is unfortunate.  When
+        // that's fixed, this code should adapt. XXX
+        snippetNode.textContent = "...";
+      } else {
+        throw e;
+      }
     }
 
     return row;
@@ -505,7 +509,12 @@ ThreadSummarizer.prototype = {
     let messageList = document.getElementById("message_list");
 
     // Remove all ignored messages from summarization.
-    let summarizedMessages = [msg for (msg of aMessages) if (!msg.isKilled)];
+    let summarizedMessages = [];
+    for (let message of aMessages) {
+      if (!message.isKilled) {
+        summarizedMessages.push(message);
+      }
+    }
     let ignoredCount = aMessages.trueLength - summarizedMessages.length;
 
     // Summarize the selected messages.
