@@ -8,12 +8,17 @@ MOZ_APP_NAME=thunderbird
 MOZ_UPDATER=1
 MOZ_THUNDERBIRD=1
 MOZ_APP_STATIC_INI=1
-MOZ_CHROME_FILE_FORMAT=omni
 MOZ_DISABLE_EXPORT_JS=1
 MOZ_NO_ACTIVEX_SUPPORT=1
 MOZ_ACTIVEX_SCRIPTING_SUPPORT=
 MOZ_LDAP_XPCOM=1
 MOZ_COMPOSER=1
+
+if test "$OS_ARCH" = "WINNT" -o \
+        "$OS_ARCH" = "Linux"; then
+  MOZ_BUNDLED_FONTS=1
+fi
+
 if test "$OS_ARCH" = "WINNT"; then
   if ! test "$HAVE_64BIT_BUILD"; then
     MOZ_VERIFY_MAR_SIGNATURE=1
@@ -22,11 +27,12 @@ if test "$OS_ARCH" = "WINNT"; then
 fi
 
 MOZ_SAFE_BROWSING=1
-MOZ_MEDIA_NAVIGATOR=1
 MOZ_MORK=1
 
 MOZ_APP_VERSION_TXT=${_topsrcdir}/$MOZ_BUILD_APP/config/version.txt
 MOZ_APP_VERSION=`cat $MOZ_APP_VERSION_TXT`
+MOZ_APP_VERSION_DISPLAY_TXT=${_topsrcdir}/$MOZ_BUILD_APP/config/version_display.txt
+MOZ_APP_VERSION_DISPLAY=`cat $MOZ_APP_VERSION_DISPLAY_TXT`
 THUNDERBIRD_VERSION=$MOZ_APP_VERSION
 
 MOZ_UA_BUILDID=20100101
@@ -45,6 +51,9 @@ JSGC_GENERATIONAL=1
 MOZ_PROFILE_MIGRATOR=1
 MOZ_JSDOWNLOADS=1
 MOZ_BINARY_EXTENSIONS=1
+MOZ_SEPARATE_MANIFEST_FOR_THEME_OVERRIDES=1
 
 # Enable building ./signmar and running libmar signature tests
 MOZ_ENABLE_SIGNMAR=1
+
+MOZ_DEVTOOLS=all
