@@ -385,15 +385,6 @@ NS_IMETHODIMP nsAbMDBDirectory::GetChildNodes(nsISimpleEnumerator* *aResult)
   return NS_NewArrayEnumerator(aResult, mSubDirectories);
 }
 
-static PLDHashOperator 
-enumerateSearchCache(nsISupports *aKey, nsCOMPtr<nsIAbCard> &aData, void* aClosure)
-{
-  nsIMutableArray* array = static_cast<nsIMutableArray*>(aClosure);
-
-  array->AppendElement(aData, false);
-  return PL_DHASH_NEXT;
-}
-
 NS_IMETHODIMP nsAbMDBDirectory::GetChildCards(nsISimpleEnumerator* *result)
 {
   nsresult rv;
@@ -407,7 +398,9 @@ NS_IMETHODIMP nsAbMDBDirectory::GetChildCards(nsISimpleEnumerator* *result)
     // Search is synchronous so need to return
     // results after search is complete
     nsCOMPtr<nsIMutableArray> array(do_CreateInstance(NS_ARRAY_CONTRACTID));
-    mSearchCache.Enumerate(enumerateSearchCache, (void*)array);
+    for (auto iter = mSearchCache.Iter(); !iter.Done(); iter.Next()) {
+      array->AppendElement(iter.Data(), false);
+    }
     return NS_NewArrayEnumerator(result, array);
   }
 
@@ -594,6 +587,22 @@ NS_IMETHODIMP nsAbMDBDirectory::HasDirectory(nsIAbDirectory *dir, bool *hasDir)
   return rv;
 }
 
+NS_IMETHODIMP nsAbMDBDirectory::HasMailListWithName(const char16_t *aName, bool *aHasList)
+{
+  NS_ENSURE_ARG_POINTER(aHasList);
+
+  nsCOMPtr<nsIAddrDatabase> database;
+  nsresult rv = GetDatabase(getter_AddRefs(database));
+  if (NS_SUCCEEDED(rv))
+  {
+    rv = database->FindMailListbyUnicodeName(aName, aHasList);
+    if (NS_SUCCEEDED(rv) && *aHasList)
+      return NS_OK;
+  }
+
+  return rv;
+}
+
 NS_IMETHODIMP nsAbMDBDirectory::AddMailList(nsIAbDirectory *list, nsIAbDirectory **addedList)
 {
   NS_ENSURE_ARG_POINTER(addedList);
