@@ -373,6 +373,13 @@ nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, nsIMsgDB
 {
   nsresult rv;
 
+  // Check for any reply type that wants to ignore the quote.
+  bool ignoreQuote = false;
+  if (type >= nsIMsgCompType::ReplyIgnoreQuote) {
+    type -= nsIMsgCompType::ReplyIgnoreQuote;
+    ignoreQuote = true;
+  }
+
   nsCOMPtr<nsIMsgIdentity> identity = aIdentity;
   if (!identity)
     GetDefaultIdentity(getter_AddRefs(identity));
@@ -406,12 +413,13 @@ nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, nsIMsgDB
       pMsgComposeParams->SetIdentity(identity);
 
       // When doing a reply (except with a template) see if there's a selection that we should quote
-      if (type == nsIMsgCompType::Reply ||
-          type == nsIMsgCompType::ReplyAll ||
-          type == nsIMsgCompType::ReplyToSender ||
-          type == nsIMsgCompType::ReplyToGroup ||
-          type == nsIMsgCompType::ReplyToSenderAndGroup ||
-          type == nsIMsgCompType::ReplyToList)
+      if (!ignoreQuote &&
+          (type == nsIMsgCompType::Reply ||
+           type == nsIMsgCompType::ReplyAll ||
+           type == nsIMsgCompType::ReplyToSender ||
+           type == nsIMsgCompType::ReplyToGroup ||
+           type == nsIMsgCompType::ReplyToSenderAndGroup ||
+           type == nsIMsgCompType::ReplyToList))
       {
         nsAutoCString selHTML;
         if (NS_SUCCEEDED(GetOrigWindowSelection(type, aMsgWindow, selHTML)))
@@ -1003,6 +1011,10 @@ nsMsgComposeService::ForwardMessage(const nsAString &forwardTo,
 
   folder->GetUriForMsg(aMsgHdr, msgUri);
 
+  nsAutoCString uriToOpen(msgUri);
+  uriToOpen += (uriToOpen.FindChar('?') == kNotFound) ? '?' : '&';
+  uriToOpen.Append("fetchCompleteMessage=true");
+
   // get the MsgIdentity for the above key using AccountManager
   nsCOMPtr<nsIMsgAccountManager> accountManager =
     do_GetService (NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
@@ -1022,10 +1034,10 @@ nsMsgComposeService::ForwardMessage(const nsAString &forwardTo,
   }
 
   if (aForwardType == nsIMsgComposeService::kForwardInline)
-    return RunMessageThroughMimeDraft(msgUri,
+    return RunMessageThroughMimeDraft(uriToOpen,
                                       nsMimeOutput::nsMimeMessageDraftOrTemplate,
                                       identity,
-                                      msgUri.get(), aMsgHdr,
+                                      uriToOpen.get(), aMsgHdr,
                                       true, forwardTo,
                                       false, aMsgWindow);
 
@@ -1049,7 +1061,7 @@ nsMsgComposeService::ForwardMessage(const nsAString &forwardTo,
   pMsgComposeParams->SetFormat(nsIMsgCompFormat::Default);
   pMsgComposeParams->SetIdentity(identity);
   pMsgComposeParams->SetComposeFields(compFields);
-  pMsgComposeParams->SetOriginalMsgURI(msgUri.get());
+  pMsgComposeParams->SetOriginalMsgURI(uriToOpen.get());
   // create the nsIMsgCompose object to send the object
   nsCOMPtr<nsIMsgCompose> pMsgCompose (do_CreateInstance(NS_MSGCOMPOSE_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
