@@ -24,17 +24,11 @@ MOZ_BRANDING_DIRECTORY=b2g/branding/unofficial
 MOZ_OFFICIAL_BRANDING_DIRECTORY=b2g/branding/official
 # MOZ_APP_DISPLAYNAME is set by branding/configure.sh
 
-MOZ_SAFE_BROWSING=1
-MOZ_SERVICES_COMMON=1
-MOZ_SERVICES_METRICS=1
 MOZ_CAPTIVEDETECT=1
 
-MOZ_WEBSMS_BACKEND=1
 MOZ_NO_SMART_CARDS=1
 MOZ_APP_STATIC_INI=1
 NSS_NO_LIBPKIX=1
-NSS_DISABLE_DBM=1
-MOZ_DISABLE_EXPORT_JS=1
 
 if test "$OS_TARGET" = "Android"; then
 MOZ_CAPTURE=1
@@ -42,22 +36,13 @@ MOZ_RAW=1
 MOZ_AUDIO_CHANNEL_MANAGER=1
 fi
 
-MOZ_MEDIA_NAVIGATOR=1
-
 MOZ_APP_ID={d1bfe7d9-c01e-4237-998b-7b5f960a4314}
 MOZ_TIME_MANAGER=1
 
-MOZ_PAY=1
 MOZ_TOOLKIT_SEARCH=
 MOZ_PLACES=
 MOZ_B2G=1
 
-if test "$OS_TARGET" = "Android"; then
-MOZ_NUWA_PROCESS=1
-MOZ_B2G_LOADER=1
-MOZ_ENABLE_WARNINGS_AS_ERRORS=1
-fi
-
 MOZ_JSDOWNLOADS=1
 
 MOZ_BUNDLED_FONTS=1
