@@ -1,8 +1,6 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
-const PRELOAD_PREF = "browser.newtab.preload";
-
 gDirectorySource = "data:application/json," + JSON.stringify({
   "directory": [{
     url: "http://example1.com/",
@@ -13,16 +11,16 @@ gDirectorySource = "data:application/json," + JSON.stringify({
   }]
 });
 
-function runTests() {
-  registerCleanupFunction(() => {
-    Services.prefs.clearUserPref(PRELOAD_PREF);
-  });
-
-  Services.prefs.setBoolPref(PRELOAD_PREF, false);
+add_task(function* () {
+  yield pushPrefs(["browser.newtab.preload", false]);
 
   // Make the page have a directory link
   yield setLinks([]);
-  yield addNewTabPageTab();
-  let titleNode = getCell(0).node.querySelector(".newtab-title");
-  is(titleNode.style.backgroundColor, "green", "title bg color is green");
-}
+  yield* addNewTabPageTab();
+
+  let color = yield performOnCell(0, cell => {
+    return cell.node.querySelector(".newtab-title").style.backgroundColor;
+  });
+
+  is(color, "green", "title bg color is green");
+});
