@@ -1,16 +1,19 @@
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
+/* vim: set ft=javascript ts=2 et sw=2 tw=80: */
 /* Any copyright is dedicated to the Public Domain.
  * http://creativecommons.org/publicdomain/zero/1.0/ */
+
 "use strict";
 
 // Test that network requests originating from the toolbox don't get recorded in
 // the network panel.
 
-add_task(function*() {
+add_task(function* () {
   // TODO: This test tries to verify the normal behavior of the netmonitor and
   // therefore needs to avoid the explicit check for tests. Bug 1167188 will
   // allow us to remove this workaround.
-  let isTesting = DevToolsUtils.testing;
-  DevToolsUtils.testing = false;
+  let isTesting = flags.testing;
+  flags.testing = false;
 
   let tab = yield addTab(URL_ROOT + "doc_viewsource.html");
   let target = TargetFactory.forTab(tab);
@@ -26,5 +29,5 @@ add_task(function*() {
   yield gDevTools.closeToolbox(target);
   tab = target = toolbox = panel = null;
   gBrowser.removeCurrentTab();
-  DevToolsUtils.testing = isTesting;
+  flags.testing = isTesting;
 });
