@@ -4,18 +4,22 @@
 
 // Tests that mosue interaction works fine with the table widget
 
+"use strict";
+
 const TEST_URI = "data:text/xml;charset=UTF-8,<?xml version='1.0'?>" +
   "<?xml-stylesheet href='chrome://global/skin/global.css'?>" +
-  "<?xml-stylesheet href='chrome://devtools/skin/common.css'?>" +
-  "<?xml-stylesheet href='chrome://devtools/skin/light-theme.css'?>" +
-  "<?xml-stylesheet href='chrome://devtools/skin/widgets.css'?>" +
+
+  // Uncomment these lines to help with visual debugging. When uncommented they
+  // dump a couple of thousand errors in the log (bug 1258285)
+  // "<?xml-stylesheet href='chrome://devtools/skin/light-theme.css'?>" +
+  // "<?xml-stylesheet href='chrome://devtools/skin/widgets.css'?>" +
+
   "<window xmlns='http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul'" +
   " title='Table Widget' width='600' height='500'>" +
   "<box flex='1' class='theme-light'/></window>";
 const TEST_OPT = "chrome,titlebar,toolbar,centerscreen,resizable,dialog=no";
 
 const {TableWidget} = require("devtools/client/shared/widgets/TableWidget");
-const Promise = require("promise");
 
 var doc, table;
 
@@ -39,6 +43,7 @@ function test() {
         emptyText: "This is dummy empty text",
         highlightUpdated: true,
         removableColumns: true,
+        wrapTextInElements: true,
       });
       startTests();
     });
@@ -52,7 +57,7 @@ function endTests() {
   finish();
 }
 
-var startTests = Task.async(function*() {
+var startTests = Task.async(function* () {
   populateTable();
   yield testMouseInteraction();
   endTests();
@@ -121,7 +126,8 @@ function populateTable() {
 // Sends a click event on the passed DOM node in an async manner
 function click(node, button = 0) {
   if (button == 0) {
-    executeSoon(() => EventUtils.synthesizeMouseAtCenter(node, {}, doc.defaultView));
+    executeSoon(() => EventUtils.synthesizeMouseAtCenter(node, {},
+                                                         doc.defaultView));
   } else {
     executeSoon(() => EventUtils.synthesizeMouseAtCenter(node, {
       button: button,
@@ -133,42 +139,58 @@ function click(node, button = 0) {
 /**
  * Tests if clicking the table items does the expected behavior
  */
-var testMouseInteraction = Task.async(function*() {
+var testMouseInteraction = Task.async(function* () {
   info("Testing mouse interaction with the table");
   ok(!table.selectedRow, "Nothing should be selected beforehand");
 
   let event = table.once(TableWidget.EVENTS.ROW_SELECTED);
-  let node = table.tbody.firstChild.firstChild.children[1];
+  let firstColumnFirstRowCell = table.tbody.firstChild.firstChild.children[1];
   info("clicking on the first row");
-  ok(!node.classList.contains("theme-selected"),
+  ok(!firstColumnFirstRowCell.classList.contains("theme-selected"),
      "Node should not have selected class before clicking");
-  click(node);
+  click(firstColumnFirstRowCell);
   let id = yield event;
-  ok(node.classList.contains("theme-selected"), "Node has selected class after click");
+  ok(firstColumnFirstRowCell.classList.contains("theme-selected"),
+     "Node has selected class after click");
   is(id, "id1", "Correct row was selected");
 
-  info("clicking on third row to select it");
+  info("clicking on second row to select it");
   event = table.once(TableWidget.EVENTS.ROW_SELECTED);
-  let node2 = table.tbody.firstChild.firstChild.children[3];
+  let firstColumnSecondRowCell = table.tbody.firstChild.firstChild.children[2];
   // node should not have selected class
-  ok(!node2.classList.contains("theme-selected"),
+  ok(!firstColumnSecondRowCell.classList.contains("theme-selected"),
      "New node should not have selected class before clicking");
-  click(node2);
+  click(firstColumnSecondRowCell);
   id = yield event;
-  ok(node2.classList.contains("theme-selected"),
+  ok(firstColumnSecondRowCell.classList.contains("theme-selected"),
      "New node has selected class after clicking");
-  is(id, "id3", "Correct table path is emitted for new node")
-  isnot(node, node2, "Old and new node are different");
-  ok(!node.classList.contains("theme-selected"),
+  is(id, "id2", "Correct table path is emitted for new node");
+  isnot(firstColumnFirstRowCell, firstColumnSecondRowCell,
+    "Old and new node are different");
+  ok(!firstColumnFirstRowCell.classList.contains("theme-selected"),
      "Old node should not have selected class after the click on new node");
 
+  info("clicking on the third row cell content to select third row");
+  event = table.once(TableWidget.EVENTS.ROW_SELECTED);
+  let firstColumnThirdRowCell = table.tbody.firstChild.firstChild.children[3];
+  let firstColumnThirdRowCellInnerNode = firstColumnThirdRowCell.querySelector("span");
+  // node should not have selected class
+  ok(!firstColumnThirdRowCell.classList.contains("theme-selected"),
+     "New node should not have selected class before clicking");
+  click(firstColumnThirdRowCellInnerNode);
+  id = yield event;
+  ok(firstColumnThirdRowCell.classList.contains("theme-selected"),
+     "New node has selected class after clicking the cell content");
+  is(id, "id3", "Correct table path is emitted for new node");
+
   // clicking on table header to sort by it
   event = table.once(TableWidget.EVENTS.COLUMN_SORTED);
-  node = table.tbody.children[6].firstChild.children[0];
+  let node = table.tbody.children[6].firstChild.children[0];
   info("clicking on the 4th coulmn header to sort the table by it");
   ok(!node.hasAttribute("sorted"),
      "Node should not have sorted attribute before clicking");
-  ok(doc.querySelector("[sorted]"), "Although, something else should be sorted on");
+  ok(doc.querySelector("[sorted]"),
+     "Although, something else should be sorted on");
   isnot(doc.querySelector("[sorted]"), node, "Which is not equal to this node");
   click(node);
   id = yield event;
@@ -182,15 +204,12 @@ var testMouseInteraction = Task.async(function*() {
   // test context menu opening.
   // hiding second column
   // event listener for popupshown
-  event = Promise.defer();
-  table.menupopup.addEventListener("popupshown", function onPopupShown(e) {
-    table.menupopup.removeEventListener("popupshown", onPopupShown);
-    event.resolve();
-  })
-  info("right clicking on the first column header");
+  info("right click on the first column header");
   node = table.tbody.firstChild.firstChild.firstChild;
+  let onPopupShown = once(table.menupopup, "popupshown");
   click(node, 2);
-  yield event.promise;
+  yield onPopupShown;
+
   is(table.menupopup.querySelectorAll("[disabled]").length, 1,
      "Only 1 menuitem is disabled");
   is(table.menupopup.querySelector("[disabled]"),
@@ -198,6 +217,7 @@ var testMouseInteraction = Task.async(function*() {
      "Which is the unique column");
   // popup should be open now
   // clicking on second column label
+  let onPopupHidden = once(table.menupopup, "popuphidden");
   event = table.once(TableWidget.EVENTS.HEADER_CONTEXT_MENU);
   node = table.menupopup.querySelector("[data-id='col2']");
   info("selecting to hide the second column");
@@ -205,25 +225,24 @@ var testMouseInteraction = Task.async(function*() {
      "Column is not hidden before hiding it");
   click(node);
   id = yield event;
+  yield onPopupHidden;
   is(id, "col2", "Correct column was triggered to be hidden");
   is(table.tbody.children[2].getAttribute("hidden"), "true",
      "Column is hidden after hiding it");
 
   // hiding third column
   // event listener for popupshown
-  event = Promise.defer();
-  table.menupopup.addEventListener("popupshown", function onPopupShown(e) {
-    table.menupopup.removeEventListener("popupshown", onPopupShown);
-    event.resolve();
-  })
   info("right clicking on the first column header");
   node = table.tbody.firstChild.firstChild.firstChild;
+  onPopupShown = once(table.menupopup, "popupshown");
   click(node, 2);
-  yield event.promise;
+  yield onPopupShown;
+
   is(table.menupopup.querySelectorAll("[disabled]").length, 1,
      "Only 1 menuitem is disabled");
   // popup should be open now
   // clicking on second column label
+  onPopupHidden = once(table.menupopup, "popuphidden");
   event = table.once(TableWidget.EVENTS.HEADER_CONTEXT_MENU);
   node = table.menupopup.querySelector("[data-id='col3']");
   info("selecting to hide the second column");
@@ -231,21 +250,19 @@ var testMouseInteraction = Task.async(function*() {
      "Column is not hidden before hiding it");
   click(node);
   id = yield event;
+  yield onPopupHidden;
   is(id, "col3", "Correct column was triggered to be hidden");
   is(table.tbody.children[4].getAttribute("hidden"), "true",
      "Column is hidden after hiding it");
 
   // opening again to see if 2 items are disabled now
   // event listener for popupshown
-  event = Promise.defer();
-  table.menupopup.addEventListener("popupshown", function onPopupShown(e) {
-    table.menupopup.removeEventListener("popupshown", onPopupShown);
-    event.resolve();
-  })
   info("right clicking on the first column header");
   node = table.tbody.firstChild.firstChild.firstChild;
+  onPopupShown = once(table.menupopup, "popupshown");
   click(node, 2);
-  yield event.promise;
+  yield onPopupShown;
+
   is(table.menupopup.querySelectorAll("[disabled]").length, 2,
      "2 menuitems are disabled now as only 2 columns remain visible");
   is(table.menupopup.querySelectorAll("[disabled]")[0],
@@ -258,6 +275,7 @@ var testMouseInteraction = Task.async(function*() {
   // showing back 2nd column
   // popup should be open now
   // clicking on second column label
+  onPopupHidden = once(table.menupopup, "popuphidden");
   event = table.once(TableWidget.EVENTS.HEADER_CONTEXT_MENU);
   node = table.menupopup.querySelector("[data-id='col2']");
   info("selecting to hide the second column");
@@ -265,23 +283,22 @@ var testMouseInteraction = Task.async(function*() {
      "Column is hidden before unhiding it");
   click(node);
   id = yield event;
+  yield onPopupHidden;
   is(id, "col2", "Correct column was triggered to be hidden");
   ok(!table.tbody.children[2].hasAttribute("hidden"),
      "Column is not hidden after unhiding it");
 
   // showing back 3rd column
   // event listener for popupshown
-  event = Promise.defer();
-  table.menupopup.addEventListener("popupshown", function onPopupShown(e) {
-    table.menupopup.removeEventListener("popupshown", onPopupShown);
-    event.resolve();
-  })
   info("right clicking on the first column header");
   node = table.tbody.firstChild.firstChild.firstChild;
+  onPopupShown = once(table.menupopup, "popupshown");
   click(node, 2);
-  yield event.promise;
+  yield onPopupShown;
+
   // popup should be open now
   // clicking on second column label
+  onPopupHidden = once(table.menupopup, "popuphidden");
   event = table.once(TableWidget.EVENTS.HEADER_CONTEXT_MENU);
   node = table.menupopup.querySelector("[data-id='col3']");
   info("selecting to hide the second column");
@@ -289,6 +306,7 @@ var testMouseInteraction = Task.async(function*() {
      "Column is hidden before unhiding it");
   click(node);
   id = yield event;
+  yield onPopupHidden;
   is(id, "col3", "Correct column was triggered to be hidden");
   ok(!table.tbody.children[4].hasAttribute("hidden"),
      "Column is not hidden after unhiding it");
