@@ -1,23 +1,25 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+/* eslint no-unused-vars: [2, {"vars": "local", "args": "none"}] */
+/* import-globals-from ../../framework/test/shared-head.js */
+
+"use strict";
 
 // shared-head.js handles imports, constants, and utility functions
 Services.scriptloader.loadSubScript("chrome://mochitests/content/browser/devtools/client/framework/test/shared-head.js", this);
 
 const {DOMHelpers} = Cu.import("resource://devtools/client/shared/DOMHelpers.jsm", {});
 const {Hosts} = require("devtools/client/framework/toolbox-hosts");
-const {defer} = require("promise");
 
 const TEST_URI_ROOT = "http://example.com/browser/devtools/client/shared/test/";
 const OPTIONS_VIEW_URL = TEST_URI_ROOT + "doc_options-view.xul";
 
 function catchFail(func) {
-  return function() {
+  return function () {
     try {
       return func.apply(null, arguments);
-    }
-    catch (ex) {
+    } catch (ex) {
       ok(false, ex);
       console.error(ex);
       finish();
@@ -29,7 +31,7 @@ function catchFail(func) {
 /**
  * Polls a given function waiting for the given value.
  *
- * @param object aOptions
+ * @param object options
  *        Options object with the following properties:
  *        - validator
  *        A validator function that should return the expected value. This is
@@ -53,49 +55,45 @@ function catchFail(func) {
  *        The expected value. If this option is omitted then the |validator|
  *        function must return a trueish value.
  *        Each of the provided callback functions will receive two arguments:
- *        the |aOptions| object and the last value returned by |validator|.
+ *        the |options| object and the last value returned by |validator|.
  */
-function waitForValue(aOptions)
-{
+function waitForValue(options) {
   let start = Date.now();
-  let timeout = aOptions.timeout || 5000;
+  let timeout = options.timeout || 5000;
   let lastValue;
 
-  function wait(validatorFn, successFn, failureFn)
-  {
+  function wait(validatorFn, successFn, failureFn) {
     if ((Date.now() - start) > timeout) {
       // Log the failure.
-      ok(false, "Timed out while waiting for: " + aOptions.name);
-      let expected = "value" in aOptions ?
-                     "'" + aOptions.value + "'" :
+      ok(false, "Timed out while waiting for: " + options.name);
+      let expected = "value" in options ?
+                     "'" + options.value + "'" :
                      "a trueish value";
       info("timeout info :: got '" + lastValue + "', expected " + expected);
-      failureFn(aOptions, lastValue);
+      failureFn(options, lastValue);
       return;
     }
 
-    lastValue = validatorFn(aOptions, lastValue);
-    let successful = "value" in aOptions ?
-                      lastValue == aOptions.value :
+    lastValue = validatorFn(options, lastValue);
+    let successful = "value" in options ?
+                      lastValue == options.value :
                       lastValue;
     if (successful) {
-      ok(true, aOptions.name);
-      successFn(aOptions, lastValue);
-    }
-    else {
+      ok(true, options.name);
+      successFn(options, lastValue);
+    } else {
       setTimeout(() => {
         wait(validatorFn, successFn, failureFn);
       }, 100);
     }
   }
 
-  wait(aOptions.validator, aOptions.success, aOptions.failure);
+  wait(options.validator, options.success, options.failure);
 }
 
 function oneTimeObserve(name, callback) {
   return new Promise((resolve) => {
-
-    var func = function() {
+    let func = function () {
       Services.obs.removeObserver(func, name);
       if (callback) {
         callback();
@@ -106,7 +104,8 @@ function oneTimeObserve(name, callback) {
   });
 }
 
-var createHost = Task.async(function*(type = "bottom", src = "data:text/html;charset=utf-8,") {
+let createHost =
+Task.async(function* (type = "bottom", src = "data:text/html;charset=utf-8,") {
   let host = new Hosts[type](gBrowser.selectedTab);
   let iframe = yield host.create();
 
@@ -119,44 +118,6 @@ var createHost = Task.async(function*(type = "bottom", src = "data:text/html;cha
   return [host, iframe.contentWindow, iframe.contentDocument];
 });
 
-/**
- * Load the Telemetry utils, then stub Telemetry.prototype.log in order to
- * record everything that's logged in it.
- * Store all recordings on Telemetry.telemetryInfo.
- * @return {Telemetry}
- */
-function loadTelemetryAndRecordLogs() {
-  info("Mock the Telemetry log function to record logged information");
-
-  let Telemetry = require("devtools/client/shared/telemetry");
-  Telemetry.prototype.telemetryInfo = {};
-  Telemetry.prototype._oldlog = Telemetry.prototype.log;
-  Telemetry.prototype.log = function(histogramId, value) {
-    if (!this.telemetryInfo) {
-      // Can be removed when Bug 992911 lands (see Bug 1011652 Comment 10)
-      return;
-    }
-    if (histogramId) {
-      if (!this.telemetryInfo[histogramId]) {
-        this.telemetryInfo[histogramId] = [];
-      }
-
-      this.telemetryInfo[histogramId].push(value);
-    }
-  };
-
-  return Telemetry;
-}
-
-/**
- * Stop recording the Telemetry logs and put back the utils as it was before.
- */
-function stopRecordingTelemetryLogs(Telemetry) {
-  Telemetry.prototype.log = Telemetry.prototype._oldlog;
-  delete Telemetry.prototype._oldlog;
-  delete Telemetry.prototype.telemetryInfo;
-}
-
 /**
  * Check the correctness of the data recorded in Telemetry after
  * loadTelemetryAndRecordLogs was called.
@@ -164,14 +125,13 @@ function stopRecordingTelemetryLogs(Telemetry) {
 function checkTelemetryResults(Telemetry) {
   let result = Telemetry.prototype.telemetryInfo;
 
-  for (let [histId, value] of Iterator(result)) {
-    if (histId.endsWith("OPENED_PER_USER_FLAG")) {
-      ok(value.length === 1 && value[0] === true,
-         "Per user value " + histId + " has a single value of true");
-    } else if (histId.endsWith("OPENED_BOOLEAN")) {
+  for (let histId in result) {
+    let value = result[histId];
+
+    if (histId.endsWith("OPENED_COUNT")) {
       ok(value.length > 1, histId + " has more than one entry");
 
-      let okay = value.every(function(element) {
+      let okay = value.every(function (element) {
         return element === true;
       });
 
@@ -179,7 +139,7 @@ function checkTelemetryResults(Telemetry) {
     } else if (histId.endsWith("TIME_ACTIVE_SECONDS")) {
       ok(value.length > 1, histId + " has more than one entry");
 
-      let okay = value.every(function(element) {
+      let okay = value.every(function (element) {
         return element > 0;
       });
 
@@ -196,10 +156,10 @@ function checkTelemetryResults(Telemetry) {
  * @param {String} toolId
  */
 function* openAndCloseToolbox(nbOfTimes, usageTime, toolId) {
-  for (let i = 0; i < nbOfTimes; i ++) {
+  for (let i = 0; i < nbOfTimes; i++) {
     info("Opening toolbox " + (i + 1));
     let target = TargetFactory.forTab(gBrowser.selectedTab);
-    yield gDevTools.showToolbox(target, toolId)
+    yield gDevTools.showToolbox(target, toolId);
 
     // We use a timeout to check the toolbox's active time
     yield new Promise(resolve => setTimeout(resolve, usageTime));
@@ -240,7 +200,7 @@ function waitUntil(predicate, interval = 10) {
     return Promise.resolve(true);
   }
   return new Promise(resolve => {
-    setTimeout(function() {
+    setTimeout(function () {
       waitUntil(predicate).then(() => resolve(true));
     }, interval);
   });
@@ -264,7 +224,8 @@ function showFilterPopupPresets(widget) {
  * @param  {string} value
  * @return {Promise}
  */
-var showFilterPopupPresetsAndCreatePreset = Task.async(function*(widget, name, value) {
+let showFilterPopupPresetsAndCreatePreset =
+Task.async(function* (widget, name, value) {
   yield showFilterPopupPresets(widget);
 
   let onRender = widget.once("render");
@@ -325,7 +286,8 @@ function checkCssSyntaxHighlighterOutput(expectedNodes, parent) {
    * Check the type and content of a single node.
    */
   function checkNode(expected, actual) {
-    ok(actual.textContent == expected.text, "Check that node has the expected textContent");
+    ok(actual.textContent == expected.text,
+       "Check that node has the expected textContent");
     info("Expected text content: [" + expected.text + "]");
     info("Actual text content: [" + actual.textContent + "]");
 
@@ -365,10 +327,10 @@ function checkCssSyntaxHighlighterOutput(expectedNodes, parent) {
   }
 
   info("Logging the actual nodes we have:");
-  for (var j = 0; j < parent.childNodes.length; j++) {
-    var n = parent.childNodes[j];
+  for (let j = 0; j < parent.childNodes.length; j++) {
+    let n = parent.childNodes[j];
     info(j + " / " +
-         "nodeType: "+ n.nodeType + " / " +
+         "nodeType: " + n.nodeType + " / " +
          "textContent: " + n.textContent);
   }
 
