@@ -1,7 +1,7 @@
-/* vim:set ts=2 sw=2 sts=2 et: */
-/* This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
+/* vim: set ft=javascript ts=2 et sw=2 tw=80: */
+/* Any copyright is dedicated to the Public Domain.
+ * http://creativecommons.org/publicdomain/zero/1.0/ */
 
 // Tests to ensure that errors don't appear when the console is closed while a
 // completion is being performed.
@@ -11,7 +11,7 @@
 const TEST_URI = "http://example.com/browser/devtools/client/webconsole/" +
                  "test/test-console.html";
 
-var test = asyncTest(function* () {
+add_task(function* () {
   let { browser } = yield loadTab(TEST_URI);
 
   let hud = yield openConsole();
@@ -21,8 +21,6 @@ var test = asyncTest(function* () {
 function testClosingAfterCompletion(hud, browser) {
   let deferred = promise.defer();
 
-  let inputNode = hud.jsterm.inputNode;
-
   let errorWhileClosing = false;
   function errorListener() {
     errorWhileClosing = true;
@@ -30,10 +28,10 @@ function testClosingAfterCompletion(hud, browser) {
 
   browser.addEventListener("error", errorListener, false);
 
-  // Focus the inputNode and perform the keycombo to close the WebConsole.
-  inputNode.focus();
+  // Focus the jsterm and perform the keycombo to close the WebConsole.
+  hud.jsterm.focus();
 
-  gDevTools.once("toolbox-destroyed", function() {
+  gDevTools.once("toolbox-destroyed", function () {
     browser.removeEventListener("error", errorListener, false);
     is(errorWhileClosing, false, "no error while closing the WebConsole");
     deferred.resolve();
