@@ -17,8 +17,9 @@ const {
   CanvasFrameAnonymousContentHelper
 } = require("devtools/server/actors/highlighters/utils/markup");
 
-add_task(function*() {
-  let doc = yield addTab("about:preferences");
+add_task(function* () {
+  let browser = yield addTab("about:preferences");
+  let doc = browser.contentDocument;
 
   let nodeBuilder = () => {
     let root = doc.createElement("div");
