@@ -4,7 +4,6 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/mozalloc.h"
-#include "nsAutoPtr.h"
 #include "nsCOMPtr.h"
 #include "nsDebug.h"
 #include "nsError.h"
@@ -22,7 +21,6 @@ nsTransactionItem::nsTransactionItem(nsITransaction *aTransaction)
 nsTransactionItem::~nsTransactionItem()
 {
   delete mRedoStack;
-
   delete mUndoStack;
 }
 
@@ -73,7 +71,6 @@ nsTransactionItem::AddChild(nsTransactionItem *aTransactionItem)
   }
 
   mUndoStack->Push(aTransactionItem);
-
   return NS_OK;
 }
 
@@ -88,34 +85,26 @@ nsresult
 nsTransactionItem::GetIsBatch(bool *aIsBatch)
 {
   NS_ENSURE_TRUE(aIsBatch, NS_ERROR_NULL_POINTER);
-
   *aIsBatch = !mTransaction;
-
   return NS_OK;
 }
 
 nsresult
 nsTransactionItem::GetNumberOfChildren(int32_t *aNumChildren)
 {
-  nsresult result;
-
   NS_ENSURE_TRUE(aNumChildren, NS_ERROR_NULL_POINTER);
 
   *aNumChildren = 0;
 
   int32_t ui = 0;
+  nsresult rv = GetNumberOfUndoItems(&ui);
+  NS_ENSURE_SUCCESS(rv, rv);
+
   int32_t ri = 0;
-
-  result = GetNumberOfUndoItems(&ui);
-
-  NS_ENSURE_SUCCESS(result, result);
-
-  result = GetNumberOfRedoItems(&ri);
-
-  NS_ENSURE_SUCCESS(result, result);
+  rv = GetNumberOfRedoItems(&ri);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   *aNumChildren = ui + ri;
-
   return NS_OK;
 }
 
@@ -127,21 +116,18 @@ nsTransactionItem::GetChild(int32_t aIndex, nsTransactionItem **aChild)
   *aChild = 0;
 
   int32_t numItems = 0;
-  nsresult result = GetNumberOfChildren(&numItems);
-
-  NS_ENSURE_SUCCESS(result, result);
-
-  if (aIndex < 0 || aIndex >= numItems)
+  nsresult rv = GetNumberOfChildren(&numItems);
+  NS_ENSURE_SUCCESS(rv, rv);
+  if (aIndex < 0 || aIndex >= numItems) {
     return NS_ERROR_FAILURE;
+  }
 
   // Children are expected to be in the order they were added,
   // so the child first added would be at the bottom of the undo
   // stack, or if there are no items on the undo stack, it would
   // be at the top of the redo stack.
-
-  result = GetNumberOfUndoItems(&numItems);
-
-  NS_ENSURE_SUCCESS(result, result);
+  rv = GetNumberOfUndoItems(&numItems);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   if (numItems > 0 && aIndex < numItems) {
     NS_ENSURE_TRUE(mUndoStack, NS_ERROR_FAILURE);
@@ -152,13 +138,10 @@ nsTransactionItem::GetChild(int32_t aIndex, nsTransactionItem **aChild)
   }
 
   // Adjust the index for the redo stack:
-
   aIndex -=  numItems;
 
-  result = GetNumberOfRedoItems(&numItems);
-
-  NS_ENSURE_SUCCESS(result, result);
-
+  rv = GetNumberOfRedoItems(&numItems);
+  NS_ENSURE_SUCCESS(rv, rv);
   NS_ENSURE_TRUE(mRedoStack && numItems != 0 && aIndex < numItems, NS_ERROR_FAILURE);
 
   RefPtr<nsTransactionItem> child = mRedoStack->GetItem(aIndex);
@@ -169,29 +152,29 @@ nsTransactionItem::GetChild(int32_t aIndex, nsTransactionItem **aChild)
 nsresult
 nsTransactionItem::DoTransaction()
 {
-  if (mTransaction)
+  if (mTransaction) {
     return mTransaction->DoTransaction();
+  }
   return NS_OK;
 }
 
 nsresult
 nsTransactionItem::UndoTransaction(nsTransactionManager *aTxMgr)
 {
-  nsresult result = UndoChildren(aTxMgr);
-
-  if (NS_FAILED(result)) {
+  nsresult rv = UndoChildren(aTxMgr);
+  if (NS_FAILED(rv)) {
     RecoverFromUndoError(aTxMgr);
-    return result;
+    return rv;
   }
 
-  if (!mTransaction)
+  if (!mTransaction) {
     return NS_OK;
+  }
 
-  result = mTransaction->UndoTransaction();
-
-  if (NS_FAILED(result)) {
+  rv = mTransaction->UndoTransaction();
+  if (NS_FAILED(rv)) {
     RecoverFromUndoError(aTxMgr);
-    return result;
+    return rv;
   }
 
   return NS_OK;
@@ -200,74 +183,64 @@ nsTransactionItem::UndoTransaction(nsTransactionManager *aTxMgr)
 nsresult
 nsTransactionItem::UndoChildren(nsTransactionManager *aTxMgr)
 {
-  RefPtr<nsTransactionItem> item;
-  nsresult result = NS_OK;
-  int32_t sz = 0;
-
   if (mUndoStack) {
     if (!mRedoStack && mUndoStack) {
       mRedoStack = new nsTransactionStack(nsTransactionStack::FOR_REDO);
     }
 
     /* Undo all of the transaction items children! */
-    sz = mUndoStack->GetSize();
+    int32_t sz = mUndoStack->GetSize();
 
+    nsresult rv = NS_OK;
     while (sz-- > 0) {
-      item = mUndoStack->Peek();
-
+      RefPtr<nsTransactionItem> item = mUndoStack->Peek();
       if (!item) {
         return NS_ERROR_FAILURE;
       }
 
       nsCOMPtr<nsITransaction> t = item->GetTransaction();
-
       bool doInterrupt = false;
-
-      result = aTxMgr->WillUndoNotify(t, &doInterrupt);
-
-      if (NS_FAILED(result)) {
-        return result;
+      rv = aTxMgr->WillUndoNotify(t, &doInterrupt);
+      if (NS_FAILED(rv)) {
+        return rv;
       }
-
       if (doInterrupt) {
         return NS_OK;
       }
 
-      result = item->UndoTransaction(aTxMgr);
-
-      if (NS_SUCCEEDED(result)) {
+      rv = item->UndoTransaction(aTxMgr);
+      if (NS_SUCCEEDED(rv)) {
         item = mUndoStack->Pop();
-        mRedoStack->Push(item);
+        mRedoStack->Push(item.forget());
       }
 
-      nsresult result2 = aTxMgr->DidUndoNotify(t, result);
-
-      if (NS_SUCCEEDED(result)) {
-        result = result2;
+      nsresult rv2 = aTxMgr->DidUndoNotify(t, rv);
+      if (NS_SUCCEEDED(rv)) {
+        rv = rv2;
       }
     }
+    // XXX NS_OK if there is no Undo items or all methods work fine, otherwise,
+    //     the result of the last item's UndoTransaction() or
+    //     DidUndoNotify() if UndoTransaction() succeeded.
+    return rv;
   }
 
-  return result;
+  return NS_OK;
 }
 
 nsresult
 nsTransactionItem::RedoTransaction(nsTransactionManager *aTxMgr)
 {
-  nsresult result;
-
   nsCOMPtr<nsITransaction> transaction(mTransaction);
   if (transaction) {
-    result = transaction->RedoTransaction();
-
-    NS_ENSURE_SUCCESS(result, result);
+    nsresult rv = transaction->RedoTransaction();
+    NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  result = RedoChildren(aTxMgr);
-
-  if (NS_FAILED(result)) {
+  nsresult rv = RedoChildren(aTxMgr);
+  if (NS_FAILED(rv)) {
     RecoverFromRedoError(aTxMgr);
-    return result;
+    return rv;
   }
 
   return NS_OK;
@@ -276,51 +249,46 @@ nsTransactionItem::RedoTransaction(nsTransactionManager *aTxMgr)
 nsresult
 nsTransactionItem::RedoChildren(nsTransactionManager *aTxMgr)
 {
-  RefPtr<nsTransactionItem> item;
-  nsresult result = NS_OK;
-
-  if (!mRedoStack)
+  if (!mRedoStack) {
     return NS_OK;
+  }
 
   /* Redo all of the transaction items children! */
   int32_t sz = mRedoStack->GetSize();
 
+  nsresult rv = NS_OK;
   while (sz-- > 0) {
-    item = mRedoStack->Peek();
-
+    RefPtr<nsTransactionItem> item = mRedoStack->Peek();
     if (!item) {
       return NS_ERROR_FAILURE;
     }
 
     nsCOMPtr<nsITransaction> t = item->GetTransaction();
-
     bool doInterrupt = false;
-
-    result = aTxMgr->WillRedoNotify(t, &doInterrupt);
-
-    if (NS_FAILED(result)) {
-      return result;
+    rv = aTxMgr->WillRedoNotify(t, &doInterrupt);
+    if (NS_FAILED(rv)) {
+      return rv;
     }
-
     if (doInterrupt) {
       return NS_OK;
     }
 
-    result = item->RedoTransaction(aTxMgr);
-
-    if (NS_SUCCEEDED(result)) {
+    rv = item->RedoTransaction(aTxMgr);
+    if (NS_SUCCEEDED(rv)) {
       item = mRedoStack->Pop();
-      mUndoStack->Push(item);
+      mUndoStack->Push(item.forget());
     }
 
-    nsresult result2 = aTxMgr->DidUndoNotify(t, result);
-
-    if (NS_SUCCEEDED(result)) {
-      result = result2;
+    // XXX Shouldn't this DidRedoNotify()? (bug 1311626)
+    nsresult rv2 = aTxMgr->DidUndoNotify(t, rv);
+    if (NS_SUCCEEDED(rv)) {
+      rv = rv2;
     }
   }
-
-  return result;
+  // XXX NS_OK if there is no Redo items or all methods work fine, otherwise,
+  //     the result of the last item's RedoTransaction() or
+  //     DidUndoNotify() if UndoTransaction() succeeded.
+  return rv;
 }
 
 nsresult
@@ -354,34 +322,27 @@ nsTransactionItem::GetNumberOfRedoItems(int32_t *aNumItems)
 nsresult
 nsTransactionItem::RecoverFromUndoError(nsTransactionManager *aTxMgr)
 {
-  //
   // If this method gets called, we never got to the point where we
   // successfully called UndoTransaction() for the transaction item itself.
   // Just redo any children that successfully called undo!
-  //
   return RedoChildren(aTxMgr);
 }
 
 nsresult
 nsTransactionItem::RecoverFromRedoError(nsTransactionManager *aTxMgr)
 {
-  //
   // If this method gets called, we already successfully called
   // RedoTransaction() for the transaction item itself. Undo all
   // the children that successfully called RedoTransaction(),
   // then undo the transaction item itself.
-  //
-
-  nsresult result;
-
-  result = UndoChildren(aTxMgr);
-
-  if (NS_FAILED(result)) {
-    return result;
+  nsresult rv = UndoChildren(aTxMgr);
+  if (NS_FAILED(rv)) {
+    return rv;
   }
 
-  if (!mTransaction)
+  if (!mTransaction) {
     return NS_OK;
+  }
 
   return mTransaction->UndoTransaction();
 }
