@@ -19,7 +19,7 @@ tag=${2-HEAD}
 cd $srcdir/libjpeg
 cp win/jsimdcfg.inc simd/
 
-revert_files="jconfig.h jconfigint.h moz.build Makefile.in MOZCHANGES mozilla.diff simd/jsimdcfg.inc"
+revert_files="1050342.diff jconfig.h jconfigint.h moz.build MOZCHANGES mozilla.diff simd/jsimdcfg.inc"
 if test -d ${topsrcdir}/.hg; then
     hg revert --no-backup $revert_files
 elif test -d ${topsrcdir}/.git; then
@@ -27,3 +27,4 @@ elif test -d ${topsrcdir}/.git; then
 fi
 
 patch -p0 -i mozilla.diff
+patch -p0 -i 1050342.diff
