@@ -88,12 +88,13 @@ nsHttpPipeline::~nsHttpPipeline()
 nsresult
 nsHttpPipeline::AddTransaction(nsAHttpTransaction *trans)
 {
-    LOG(("nsHttpPipeline::AddTransaction [this=%p trans=%x]\n", this, trans));
+    LOG(("nsHttpPipeline::AddTransaction [this=%p trans=%p]\n", this, trans));
 
     if (mRequestQ.Length() || mResponseQ.Length())
         mUtilizedPipeline = true;
 
-    NS_ADDREF(trans);
+    // A reference to the actual transaction is held by the pipeline transaction
+    // in either the request or response queue
     mRequestQ.AppendElement(trans);
     uint32_t qlen = PipelineDepth();
 
@@ -184,6 +185,10 @@ nsHttpPipeline::OnHeadersAvailable(nsAHttpTransaction *trans,
     GetConnectionInfo(getter_AddRefs(ci));
     MOZ_ASSERT(ci);
 
+    if (!ci) {
+        return NS_ERROR_UNEXPECTED;
+    }
+
     bool pipeliningBefore = gHttpHandler->ConnMgr()->SupportsPipelining(ci);
 
     // trans has now received its response headers; forward to the real connection
@@ -192,41 +197,38 @@ nsHttpPipeline::OnHeadersAvailable(nsAHttpTransaction *trans,
                                                   responseHead,
                                                   reset);
 
-    if (!pipeliningBefore && gHttpHandler->ConnMgr()->SupportsPipelining(ci))
+    if (!pipeliningBefore && gHttpHandler->ConnMgr()->SupportsPipelining(ci)) {
         // The received headers have expanded the eligible
         // pipeline depth for this connection
         gHttpHandler->ConnMgr()->ProcessPendingQForEntry(ci);
+    }
 
     return rv;
 }
 
 void
-nsHttpPipeline::CloseTransaction(nsAHttpTransaction *trans, nsresult reason)
+nsHttpPipeline::CloseTransaction(nsAHttpTransaction *aTrans, nsresult reason)
 {
-    LOG(("nsHttpPipeline::CloseTransaction [this=%p trans=%x reason=%x]\n",
-        this, trans, reason));
+    LOG(("nsHttpPipeline::CloseTransaction [this=%p trans=%p reason=%x]\n",
+        this, aTrans, reason));
 
     MOZ_ASSERT(PR_GetCurrentThread() == gSocketThread);
     MOZ_ASSERT(NS_FAILED(reason), "expecting failure code");
 
     // the specified transaction is to be closed with the given "reason"
-
+    RefPtr<nsAHttpTransaction> trans(aTrans);
     int32_t index;
     bool killPipeline = false;
 
-    index = mRequestQ.IndexOf(trans);
-    if (index >= 0) {
+    if ((index = mRequestQ.IndexOf(trans)) >= 0) {
         if (index == 0 && mRequestIsPartial) {
             // the transaction is in the request queue.  check to see if any of
             // its data has been written out yet.
             killPipeline = true;
         }
         mRequestQ.RemoveElementAt(index);
-    }
-    else {
-        index = mResponseQ.IndexOf(trans);
-        if (index >= 0)
-            mResponseQ.RemoveElementAt(index);
+    } else if ((index = mResponseQ.IndexOf(trans)) >= 0) {
+        mResponseQ.RemoveElementAt(index);
         // while we could avoid killing the pipeline if this transaction is the
         // last transaction in the pipeline, there doesn't seem to be that much
         // value in doing so.  most likely if this transaction is going away,
@@ -239,7 +241,7 @@ nsHttpPipeline::CloseTransaction(nsAHttpTransaction *trans, nsresult reason)
     DontReuse();
 
     trans->Close(reason);
-    NS_RELEASE(trans);
+    trans = nullptr;
 
     if (killPipeline) {
         // reschedule anything from this pipeline onto a different connection
@@ -326,7 +328,7 @@ nsHttpPipeline::PushBack(const char *data, uint32_t length)
     return NS_OK;
 }
 
-nsHttpConnection *
+already_AddRefed<nsHttpConnection>
 nsHttpPipeline::TakeHttpConnection()
 {
     if (mConnection)
@@ -382,11 +384,10 @@ nsHttpPipeline::TakeSubTransactions(
     int32_t i, count = mRequestQ.Length();
     for (i = 0; i < count; ++i) {
         nsAHttpTransaction *trans = Request(i);
-        // set the transaction conneciton object back to the underlying
+        // set the transaction connection object back to the underlying
         // nsHttpConnectionHandle
         trans->SetConnection(mConnection);
         outTransactions.AppendElement(trans);
-        NS_RELEASE(trans);
     }
     mRequestQ.Clear();
 
@@ -401,7 +402,7 @@ nsHttpPipeline::TakeSubTransactions(
 void
 nsHttpPipeline::SetConnection(nsAHttpConnection *conn)
 {
-    LOG(("nsHttpPipeline::SetConnection [this=%p conn=%x]\n", this, conn));
+    LOG(("nsHttpPipeline::SetConnection [this=%p conn=%p]\n", this, conn));
 
     MOZ_ASSERT(PR_GetCurrentThread() == gSocketThread);
     MOZ_ASSERT(!conn || !mConnection, "already have a connection");
@@ -412,7 +413,7 @@ nsHttpPipeline::SetConnection(nsAHttpConnection *conn)
 nsAHttpConnection *
 nsHttpPipeline::Connection()
 {
-    LOG(("nsHttpPipeline::Connection [this=%p conn=%x]\n", this, mConnection.get()));
+    LOG(("nsHttpPipeline::Connection [this=%p conn=%p]\n", this, mConnection.get()));
 
     MOZ_ASSERT(PR_GetCurrentThread() == gSocketThread);
     return mConnection;
@@ -589,7 +590,7 @@ nsHttpPipeline::Available()
     return result;
 }
 
-NS_METHOD
+nsresult
 nsHttpPipeline::ReadFromPipe(nsIInputStream *stream,
                              void *closure,
                              const char *buf,
@@ -681,8 +682,7 @@ nsHttpPipeline::WriteSegments(nsAHttpSegmentWriter *writer,
             rv = NS_BASE_STREAM_WOULD_BLOCK;
         else
             rv = NS_BASE_STREAM_CLOSED;
-    }
-    else {
+    } else {
         //
         // ask the transaction to consume data from the connection.
         // PushBack may be called recursively.
@@ -694,7 +694,6 @@ nsHttpPipeline::WriteSegments(nsAHttpSegmentWriter *writer,
 
             // Release the transaction if it is not IsProxyConnectInProgress()
             if (trans == Response(0)) {
-                NS_RELEASE(trans);
                 mResponseQ.RemoveElementAt(0);
                 mResponseIsPartial = false;
                 ++mHttp1xTransactionCount;
@@ -712,7 +711,7 @@ nsHttpPipeline::WriteSegments(nsAHttpSegmentWriter *writer,
     }
 
     if (mPushBackLen) {
-        nsHttpPushBackWriter writer(mPushBackBuf, mPushBackLen);
+        nsHttpPushBackWriter pushBackWriter(mPushBackBuf, mPushBackLen);
         uint32_t len = mPushBackLen, n;
         mPushBackLen = 0;
 
@@ -727,7 +726,7 @@ nsHttpPipeline::WriteSegments(nsAHttpSegmentWriter *writer,
         // the push back buffer is never larger than NS_HTTP_SEGMENT_SIZE,
         // so we are guaranteed that the next response will eat the entire
         // push back buffer (even though it might again call PushBack).
-        rv = WriteSegments(&writer, len, &n);
+        rv = WriteSegments(&pushBackWriter, len, &n);
     }
 
     return rv;
@@ -758,7 +757,6 @@ nsHttpPipeline::CancelPipeline(nsresult originalReason)
             trans->Close(originalReason);
         else
             trans->Close(NS_ERROR_NET_RESET);
-        NS_RELEASE(trans);
     }
     mRequestQ.Clear();
 
@@ -769,7 +767,6 @@ nsHttpPipeline::CancelPipeline(nsresult originalReason)
     for (i = 1; i < respLen; ++i) {
         trans = Response(i);
         trans->Close(NS_ERROR_NET_RESET);
-        NS_RELEASE(trans);
     }
 
     if (respLen > 1)
@@ -825,7 +822,6 @@ nsHttpPipeline::Close(nsresult reason)
         trans->Close(reason);
     }
 
-    NS_RELEASE(trans);
     mResponseQ.Clear();
 }
 
@@ -857,7 +853,7 @@ nsHttpPipeline::FillSendBuf()
 
     uint32_t n;
     uint64_t avail;
-    nsAHttpTransaction *trans;
+    RefPtr<nsAHttpTransaction> trans;
     nsITransport *transport = Transport();
 
     while ((trans = Request(0)) != nullptr) {
