@@ -1,5 +1,4 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* This Source Code Form is subject to the terms of the Mozilla Public
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- *//* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
@@ -22,9 +21,11 @@
 
 #include "nsIDOMWindow.h"
 #include "nsGlobalWindow.h"
+#include "nsRefreshDriver.h"
 
-#include "mozilla/unused.h"
+#include "mozilla/Unused.h"
 #include "mozilla/ArrayUtils.h"
+#include "mozilla/EventStateManager.h"
 #include "mozilla/Services.h"
 #include "mozilla/Telemetry.h"
 
@@ -60,19 +61,19 @@ namespace {
  * @return null if the code is not executed in a window or in
  * case of error, a nsPIDOMWindow otherwise.
  */
-already_AddRefed<nsPIDOMWindow>
+already_AddRefed<nsPIDOMWindowOuter>
 GetPrivateWindow(JSContext* cx) {
-  nsCOMPtr<nsPIDOMWindow> win = xpc::CurrentWindowOrNull(cx);
+  nsGlobalWindow* win = xpc::CurrentWindowOrNull(cx);
   if (!win) {
     return nullptr;
   }
 
-  win = win->GetOuterWindow();
-  if (!win) {
+  nsPIDOMWindowOuter* outer = win->AsInner()->GetOuterWindow();
+  if (!outer) {
     return nullptr;
   }
 
-  nsCOMPtr<nsPIDOMWindow> top = win->GetTop();
+  nsCOMPtr<nsPIDOMWindowOuter> top = outer->GetTop();
   if (!top) {
     return nullptr;
   }
@@ -124,17 +125,24 @@ CompartmentName(JSContext* cx, JS::Handle<JSObject*> global, nsAString& name) {
  * Generate a unique-to-the-application identifier for a group.
  */
 void
-GenerateUniqueGroupId(const JSRuntime* rt, uint64_t uid, uint64_t processId, nsAString& groupId) {
-  uint64_t runtimeId = reinterpret_cast<uintptr_t>(rt);
+GenerateUniqueGroupId(const JSContext* cx, uint64_t uid, uint64_t processId, nsAString& groupId) {
+  uint64_t contextId = reinterpret_cast<uintptr_t>(cx);
 
   groupId.AssignLiteral("process: ");
   groupId.AppendInt(processId);
   groupId.AppendLiteral(", thread: ");
-  groupId.AppendInt(runtimeId);
+  groupId.AppendInt(contextId);
   groupId.AppendLiteral(", group: ");
   groupId.AppendInt(uid);
 }
 
+static const char* TOPICS[] = {
+  "profile-before-change",
+  "quit-application",
+  "quit-application-granted",
+  "xpcom-will-shutdown"
+};
+
 } // namespace
 
 /* ------------------------------------------------------
@@ -164,7 +172,9 @@ nsPerformanceObservationTarget::SetTarget(nsPerformanceGroupDetails* details) {
 
 NS_IMETHODIMP
 nsPerformanceObservationTarget::AddJankObserver(nsIPerformanceObserver* observer) {
-  mObservers.append(observer);
+  if (!mObservers.append(observer)) {
+    MOZ_CRASH();
+  }
   return NS_OK;
 };
 
@@ -188,7 +198,9 @@ void
 nsPerformanceObservationTarget::NotifyJankObservers(nsIPerformanceGroupDetails* source, nsIPerformanceAlert* gravity) {
   // Copy the vector to make sure that it won't change under our feet.
   mozilla::Vector<nsCOMPtr<nsIPerformanceObserver>> observers;
-  observers.appendAll(mObservers);
+  if (!observers.appendAll(mObservers)) {
+    MOZ_CRASH();
+  }
 
   // Now actually notify.
   for (auto iter = observers.begin(), end = observers.end(); iter < end; ++iter) {
@@ -543,7 +555,6 @@ PerformanceAlert::GetReason(uint32_t* result) {
   *result = mReason;
   return NS_OK;
 }
-
 /* ------------------------------------------------------
  *
  * class PendingAlertsCollector
@@ -559,13 +570,14 @@ public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSITIMERCALLBACK
 
-  PendingAlertsCollector(JSRuntime* runtime, nsPerformanceStatsService* service)
+  explicit PendingAlertsCollector(nsPerformanceStatsService* service)
     : mService(service)
     , mPending(false)
   { }
 
   nsresult Start(uint32_t timerDelayMS);
   nsresult Dispose();
+
 private:
   ~PendingAlertsCollector() {}
 
@@ -573,6 +585,8 @@ private:
   bool mPending;
 
   nsCOMPtr<nsITimer> mTimer;
+
+  mozilla::Vector<uint64_t> mJankLevels;
 };
 
 NS_IMPL_ISUPPORTS(PendingAlertsCollector, nsITimerCallback);
@@ -580,7 +594,7 @@ NS_IMPL_ISUPPORTS(PendingAlertsCollector, nsITimerCallback);
 NS_IMETHODIMP
 PendingAlertsCollector::Notify(nsITimer*) {
   mPending = false;
-  mService->NotifyJankObservers();
+  mService->NotifyJankObservers(mJankLevels);
   return NS_OK;
 }
 
@@ -601,6 +615,11 @@ PendingAlertsCollector::Start(uint32_t timerDelayMS) {
   }
 
   mPending = true;
+  {
+    mozilla::DebugOnly<bool> result = nsRefreshDriver::GetJankLevels(mJankLevels);
+    MOZ_ASSERT(result);
+  }
+
   return NS_OK;
 }
 
@@ -626,14 +645,15 @@ NS_IMPL_ISUPPORTS(nsPerformanceStatsService, nsIPerformanceStatsService, nsIObse
 
 nsPerformanceStatsService::nsPerformanceStatsService()
   : mIsAvailable(false)
+  , mDisposed(false)
 #if defined(XP_WIN)
   , mProcessId(GetCurrentProcessId())
 #else
   , mProcessId(getpid())
 #endif
-  , mRuntime(xpc::GetJSRuntime())
+  , mContext(mozilla::dom::danger::GetJSContext())
   , mUIdCounter(0)
-  , mTopGroup(nsPerformanceGroup::Make(mRuntime,
+  , mTopGroup(nsPerformanceGroup::Make(mContext,
                                        this,
                                        NS_LITERAL_STRING("<process>"), // name
                                        NS_LITERAL_STRING(""),          // addonid
@@ -642,18 +662,21 @@ nsPerformanceStatsService::nsPerformanceStatsService()
                                        true, // isSystem
                                        nsPerformanceGroup::GroupScope::RUNTIME // scope
                                      ))
+  , mIsHandlingUserInput(false)
   , mProcessStayed(0)
   , mProcessMoved(0)
   , mProcessUpdateCounter(0)
   , mIsMonitoringPerCompartment(false)
   , mJankAlertThreshold(mozilla::MaxValue<uint64_t>::value) // By default, no alerts
   , mJankAlertBufferingDelay(1000 /* ms */)
+  , mJankLevelVisibilityThreshold(/* 2 ^ */ 8 /* ms */)
+  , mMaxExpectedDurationOfInteractionUS(150 * 1000)
 {
-  mPendingAlertsCollector = new PendingAlertsCollector(mRuntime, this);
+  mPendingAlertsCollector = new PendingAlertsCollector(this);
 
   // Attach some artificial group information to the universal listeners, to aid with debugging.
   nsString groupIdForAddons;
-  GenerateUniqueGroupId(mRuntime, GetNextId(), mProcessId, groupIdForAddons);
+  GenerateUniqueGroupId(mContext, GetNextId(), mProcessId, groupIdForAddons);
   mUniversalTargets.mAddons->
     SetTarget(new nsPerformanceGroupDetails(NS_LITERAL_STRING("<universal add-on listener>"),
                                             groupIdForAddons,
@@ -664,7 +687,7 @@ nsPerformanceStatsService::nsPerformanceStatsService()
 
 
   nsString groupIdForWindows;
-  GenerateUniqueGroupId(mRuntime, GetNextId(), mProcessId, groupIdForWindows);
+  GenerateUniqueGroupId(mContext, GetNextId(), mProcessId, groupIdForWindows);
   mUniversalTargets.mWindows->
     SetTarget(new nsPerformanceGroupDetails(NS_LITERAL_STRING("<universal window listener>"),
                                             groupIdForWindows,
@@ -690,25 +713,30 @@ nsPerformanceStatsService::Dispose()
   RefPtr<nsPerformanceStatsService> kungFuDeathGrip(this);
   mIsAvailable = false;
 
+  if (mDisposed) {
+    // Make sure that we don't double-dispose.
+    return;
+  }
+  mDisposed = true;
+
   // Disconnect from nsIObserverService.
   nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
   if (obs) {
-    obs->RemoveObserver(this, "profile-before-change");
-    obs->RemoveObserver(this, "quit-application");
-    obs->RemoveObserver(this, "quit-application-granted");
-    obs->RemoveObserver(this, "content-child-shutdown");
-    obs->RemoveObserver(this, "xpcom-will-shutdown");
+    for (size_t i = 0; i < mozilla::ArrayLength(TOPICS); ++i) {
+      mozilla::Unused << obs->RemoveObserver(this, TOPICS[i]);
+    }
   }
 
   // Clear up and disconnect from JSAPI.
-  js::DisposePerformanceMonitoring(mRuntime);
+  JSContext* cx = mContext;
+  js::DisposePerformanceMonitoring(cx);
 
-  mozilla::Unused << js::SetStopwatchIsMonitoringCPOW(mRuntime, false);
-  mozilla::Unused << js::SetStopwatchIsMonitoringJank(mRuntime, false);
+  mozilla::Unused << js::SetStopwatchIsMonitoringCPOW(cx, false);
+  mozilla::Unused << js::SetStopwatchIsMonitoringJank(cx, false);
 
-  mozilla::Unused << js::SetStopwatchStartCallback(mRuntime, nullptr, nullptr);
-  mozilla::Unused << js::SetStopwatchCommitCallback(mRuntime, nullptr, nullptr);
-  mozilla::Unused << js::SetGetPerformanceGroupsCallback(mRuntime, nullptr, nullptr);
+  mozilla::Unused << js::SetStopwatchStartCallback(cx, nullptr, nullptr);
+  mozilla::Unused << js::SetStopwatchCommitCallback(cx, nullptr, nullptr);
+  mozilla::Unused << js::SetGetPerformanceGroupsCallback(cx, nullptr, nullptr);
 
   // Clear up and disconnect the alerts collector.
   if (mPendingAlertsCollector) {
@@ -733,7 +761,9 @@ nsPerformanceStatsService::Dispose()
   // not modify the hashtable while iterating it.
   GroupVector groups;
   for (auto iter = mGroups.Iter(); !iter.Done(); iter.Next()) {
-    groups.append(iter.Get()->GetKey());
+    if (!groups.append(iter.Get()->GetKey())) {
+      MOZ_CRASH();
+    }
   }
   for (auto iter = groups.begin(), end = groups.end(); iter < end; ++iter) {
     RefPtr<nsPerformanceGroup> group = *iter;
@@ -764,21 +794,20 @@ nsPerformanceStatsService::InitInternal()
   // regular shutdown order.
   nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
   if (obs) {
-    obs->AddObserver(this, "profile-before-change", false);
-    obs->AddObserver(this, "quit-application-granted", false);
-    obs->AddObserver(this, "quit-application", false);
-    obs->AddObserver(this, "content-child-shutdown", false);
-    obs->AddObserver(this, "xpcom-will-shutdown", false);
+    for (size_t i = 0; i < mozilla::ArrayLength(TOPICS); ++i) {
+      mozilla::Unused << obs->AddObserver(this, TOPICS[i], false);
+    }
   }
 
   // Connect to JSAPI.
-  if (!js::SetStopwatchStartCallback(mRuntime, StopwatchStartCallback, this)) {
+  JSContext* cx = mContext;
+  if (!js::SetStopwatchStartCallback(cx, StopwatchStartCallback, this)) {
     return NS_ERROR_UNEXPECTED;
   }
-  if (!js::SetStopwatchCommitCallback(mRuntime, StopwatchCommitCallback, this)) {
+  if (!js::SetStopwatchCommitCallback(cx, StopwatchCommitCallback, this)) {
     return NS_ERROR_UNEXPECTED;
   }
-  if (!js::SetGetPerformanceGroupsCallback(mRuntime, GetPerformanceGroupsCallback, this)) {
+  if (!js::SetGetPerformanceGroupsCallback(cx, GetPerformanceGroupsCallback, this)) {
     return NS_ERROR_UNEXPECTED;
   }
 
@@ -796,13 +825,21 @@ nsPerformanceStatsService::Observe(nsISupports *aSubject, const char *aTopic,
   MOZ_ASSERT(strcmp(aTopic, "profile-before-change") == 0
              || strcmp(aTopic, "quit-application") == 0
              || strcmp(aTopic, "quit-application-granted") == 0
-             || strcmp(aTopic, "content-child-shutdown") == 0
              || strcmp(aTopic, "xpcom-will-shutdown") == 0);
 
   Dispose();
   return NS_OK;
 }
 
+/*static*/ bool
+nsPerformanceStatsService::IsHandlingUserInput() {
+  if (mozilla::EventStateManager::LatestUserInputStart().IsNull()) {
+    return false;
+  }
+  bool result = mozilla::TimeStamp::Now() - mozilla::EventStateManager::LatestUserInputStart() <= mozilla::TimeDuration::FromMicroseconds(mMaxExpectedDurationOfInteractionUS);
+  return result;
+}
+
 /* [implicit_jscontext] attribute bool isMonitoringCPOW; */
 NS_IMETHODIMP
 nsPerformanceStatsService::GetIsMonitoringCPOW(JSContext* cx, bool *aIsStopwatchActive)
@@ -811,8 +848,7 @@ nsPerformanceStatsService::GetIsMonitoringCPOW(JSContext* cx, bool *aIsStopwatch
     return NS_ERROR_NOT_AVAILABLE;
   }
 
-  JSRuntime *runtime = JS_GetRuntime(cx);
-  *aIsStopwatchActive = js::GetStopwatchIsMonitoringCPOW(runtime);
+  *aIsStopwatchActive = js::GetStopwatchIsMonitoringCPOW(cx);
   return NS_OK;
 }
 NS_IMETHODIMP
@@ -822,8 +858,7 @@ nsPerformanceStatsService::SetIsMonitoringCPOW(JSContext* cx, bool aIsStopwatchA
     return NS_ERROR_NOT_AVAILABLE;
   }
 
-  JSRuntime *runtime = JS_GetRuntime(cx);
-  if (!js::SetStopwatchIsMonitoringCPOW(runtime, aIsStopwatchActive)) {
+  if (!js::SetStopwatchIsMonitoringCPOW(cx, aIsStopwatchActive)) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
   return NS_OK;
@@ -837,8 +872,7 @@ nsPerformanceStatsService::GetIsMonitoringJank(JSContext* cx, bool *aIsStopwatch
     return NS_ERROR_NOT_AVAILABLE;
   }
 
-  JSRuntime *runtime = JS_GetRuntime(cx);
-  *aIsStopwatchActive = js::GetStopwatchIsMonitoringJank(runtime);
+  *aIsStopwatchActive = js::GetStopwatchIsMonitoringJank(cx);
   return NS_OK;
 }
 NS_IMETHODIMP
@@ -848,8 +882,7 @@ nsPerformanceStatsService::SetIsMonitoringJank(JSContext* cx, bool aIsStopwatchA
     return NS_ERROR_NOT_AVAILABLE;
   }
 
-  JSRuntime *runtime = JS_GetRuntime(cx);
-  if (!js::SetStopwatchIsMonitoringJank(runtime, aIsStopwatchActive)) {
+  if (!js::SetStopwatchIsMonitoringJank(cx, aIsStopwatchActive)) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
   return NS_OK;
@@ -975,7 +1008,7 @@ nsPerformanceStatsService::GetSnapshot(JSContext* cx, nsIPerformanceSnapshot * *
     }
   }
 
-  js::GetPerfMonitoringTestCpuRescheduling(JS_GetRuntime(cx), &mProcessStayed, &mProcessMoved);
+  js::GetPerfMonitoringTestCpuRescheduling(cx, &mProcessStayed, &mProcessMoved);
 
   if (++mProcessUpdateCounter % 10 == 0) {
     mozilla::Unused << UpdateTelemetry();
@@ -992,13 +1025,18 @@ nsPerformanceStatsService::GetNextId() {
 }
 
 /* static*/ bool
-nsPerformanceStatsService::GetPerformanceGroupsCallback(JSContext* cx, JSGroupVector& out, void* closure) {
+nsPerformanceStatsService::GetPerformanceGroupsCallback(JSContext* cx,
+                                                        js::PerformanceGroupVector& out,
+                                                        void* closure)
+{
   RefPtr<nsPerformanceStatsService> self = reinterpret_cast<nsPerformanceStatsService*>(closure);
   return self->GetPerformanceGroups(cx, out);
 }
 
 bool
-nsPerformanceStatsService::GetPerformanceGroups(JSContext* cx, JSGroupVector& out) {
+nsPerformanceStatsService::GetPerformanceGroups(JSContext* cx,
+                                                js::PerformanceGroupVector& out)
+{
   JS::RootedObject global(cx, JS::CurrentGlobalOrNull(cx));
   if (!global) {
     // While it is possible for a compartment to have no global
@@ -1007,7 +1045,10 @@ nsPerformanceStatsService::GetPerformanceGroups(JSContext* cx, JSGroupVector& ou
   }
 
   // All compartments belong to the top group.
-  out.append(mTopGroup);
+  if (!out.append(mTopGroup)) {
+    JS_ReportOutOfMemory(cx);
+    return false;
+  }
 
   nsAutoString name;
   CompartmentName(cx, global, name);
@@ -1026,20 +1067,22 @@ nsPerformanceStatsService::GetPerformanceGroups(JSContext* cx, JSGroupVector& ou
       addonName.Append(addonId);
       addonName.AppendLiteral(")");
       entry->
-        SetGroup(nsPerformanceGroup::Make(mRuntime, this,
+        SetGroup(nsPerformanceGroup::Make(mContext, this,
                                           addonName, addonId, 0,
                                           mProcessId, isSystem,
                                           nsPerformanceGroup::GroupScope::ADDON)
                  );
     }
-    out.append(entry->GetGroup());
+    if (!out.append(entry->GetGroup())) {
+      JS_ReportOutOfMemory(cx);
+      return false;
+    }
   }
 
   // Find out if the compartment is executed by a window. If so, its
   // duration should count towards the total duration of the window.
-  nsCOMPtr<nsPIDOMWindow> ptop = GetPrivateWindow(cx);
   uint64_t windowId = 0;
-  if (ptop) {
+  if (nsCOMPtr<nsPIDOMWindowOuter> ptop = GetPrivateWindow(cx)) {
     windowId = ptop->WindowID();
     auto entry = mWindowIdToGroup.PutEntry(windowId);
     if (!entry->GetGroup()) {
@@ -1048,22 +1091,28 @@ nsPerformanceStatsService::GetPerformanceGroups(JSContext* cx, JSGroupVector& ou
       windowName.AppendInt(windowId);
       windowName.AppendLiteral(")");
       entry->
-        SetGroup(nsPerformanceGroup::Make(mRuntime, this,
+        SetGroup(nsPerformanceGroup::Make(mContext, this,
                                           windowName, EmptyString(), windowId,
                                           mProcessId, isSystem,
                                           nsPerformanceGroup::GroupScope::WINDOW)
                  );
     }
-    out.append(entry->GetGroup());
+    if (!out.append(entry->GetGroup())) {
+      JS_ReportOutOfMemory(cx);
+      return false;
+    }
   }
 
   // All compartments have their own group.
   auto group =
-    nsPerformanceGroup::Make(mRuntime, this,
+    nsPerformanceGroup::Make(mContext, this,
                              name, addonId, windowId,
                              mProcessId, isSystem,
                              nsPerformanceGroup::GroupScope::COMPARTMENT);
-  out.append(group);
+  if (!out.append(group)) {
+    JS_ReportOutOfMemory(cx);
+    return false;
+  }
 
   return true;
 }
@@ -1078,6 +1127,9 @@ bool
 nsPerformanceStatsService::StopwatchStart(uint64_t iteration) {
   mIteration = iteration;
 
+  mIsHandlingUserInput = IsHandlingUserInput();
+  mUserInputCount = mozilla::EventStateManager::UserInputCount();
+
   nsresult rv = GetResources(&mUserTimeStart, &mSystemTimeStart);
   if (NS_FAILED(rv)) {
     return false;
@@ -1087,13 +1139,17 @@ nsPerformanceStatsService::StopwatchStart(uint64_t iteration) {
 }
 
 /*static*/ bool
-nsPerformanceStatsService::StopwatchCommitCallback(uint64_t iteration, JSGroupVector& recentGroups, void* closure) {
+nsPerformanceStatsService::StopwatchCommitCallback(uint64_t iteration,
+                                                   js::PerformanceGroupVector& recentGroups,
+                                                   void* closure)
+{
   RefPtr<nsPerformanceStatsService> self = reinterpret_cast<nsPerformanceStatsService*>(closure);
   return self->StopwatchCommit(iteration, recentGroups);
 }
 
 bool
-nsPerformanceStatsService::StopwatchCommit(uint64_t iteration, JSGroupVector& recentGroups)
+nsPerformanceStatsService::StopwatchCommit(uint64_t iteration,
+                                           js::PerformanceGroupVector& recentGroups)
 {
   MOZ_ASSERT(iteration == mIteration);
   MOZ_ASSERT(!recentGroups.empty());
@@ -1117,12 +1173,13 @@ nsPerformanceStatsService::StopwatchCommit(uint64_t iteration, JSGroupVector& re
   MOZ_ASSERT(mTopGroup->isUsedInThisIteration());
   const uint64_t totalRecentCycles = mTopGroup->recentCycles(iteration);
 
+  const bool isHandlingUserInput = mIsHandlingUserInput || mozilla::EventStateManager::UserInputCount() > mUserInputCount;
 
   // We should only reach this stage if `group` has had some activity.
   MOZ_ASSERT(mTopGroup->recentTicks(iteration) > 0);
   for (auto iter = recentGroups.begin(), end = recentGroups.end(); iter != end; ++iter) {
     RefPtr<nsPerformanceGroup> group = nsPerformanceGroup::Get(*iter);
-    CommitGroup(iteration, userTimeDelta, systemTimeDelta, totalRecentCycles, group);
+    CommitGroup(iteration, userTimeDelta, systemTimeDelta, totalRecentCycles, isHandlingUserInput, group);
   }
 
   // Make sure that `group` was treated along with the other items of `recentGroups`.
@@ -1139,7 +1196,9 @@ nsPerformanceStatsService::StopwatchCommit(uint64_t iteration, JSGroupVector& re
 void
 nsPerformanceStatsService::CommitGroup(uint64_t iteration,
                                        uint64_t totalUserTimeDelta, uint64_t totalSystemTimeDelta,
-                                       uint64_t totalCyclesDelta, nsPerformanceGroup* group) {
+                                       uint64_t totalCyclesDelta,
+                                       bool isHandlingUserInput,
+                                       nsPerformanceGroup* group) {
 
   MOZ_ASSERT(group->isUsedInThisIteration());
 
@@ -1187,11 +1246,15 @@ nsPerformanceStatsService::CommitGroup(uint64_t iteration,
 
   group->RecordJank(totalTimeDelta);
   group->RecordCPOW(cpowTimeDelta);
+  if (isHandlingUserInput) {
+    group->RecordUserInput();
+  }
 
   if (totalTimeDelta >= mJankAlertThreshold) {
     if (!group->HasPendingAlert()) {
-      group->SetHasPendingAlert(true);
-      mPendingAlerts.append(group);
+      if (mPendingAlerts.append(group)) {
+        group->SetHasPendingAlert(true);
+      }
       return;
     }
   }
@@ -1278,7 +1341,7 @@ nsPerformanceStatsService::GetResources(uint64_t* userTime,
 }
 
 void
-nsPerformanceStatsService::NotifyJankObservers() {
+nsPerformanceStatsService::NotifyJankObservers(const mozilla::Vector<uint64_t>& aPreviousJankLevels) {
   GroupVector alerts;
   mPendingAlerts.swap(alerts);
   if (!mPendingAlertsCollector) {
@@ -1286,10 +1349,32 @@ nsPerformanceStatsService::NotifyJankObservers() {
     return;
   }
 
+  // Find out if we have noticed any user-noticeable delay in an
+  // animation recently (i.e. since the start of the execution of JS
+  // code that caused this collector to start). If so, we'll mark any
+  // alert as part of a user-noticeable jank. Note that this doesn't
+  // mean with any certainty that the alert is the only cause of jank,
+  // or even the main cause of jank.
+  mozilla::Vector<uint64_t> latestJankLevels;
+  {
+    mozilla::DebugOnly<bool> result = nsRefreshDriver::GetJankLevels(latestJankLevels);
+    MOZ_ASSERT(result);
+  }
+  MOZ_ASSERT(latestJankLevels.length() == aPreviousJankLevels.length());
+
+  bool isJankInAnimation = false;
+  for (size_t i = mJankLevelVisibilityThreshold; i < latestJankLevels.length(); ++i) {
+    if (latestJankLevels[i] > aPreviousJankLevels[i]) {
+      isJankInAnimation = true;
+      break;
+    }
+  }
+
   MOZ_ASSERT(!alerts.empty());
   const bool hasUniversalAddonObservers = mUniversalTargets.mAddons->HasObservers();
   const bool hasUniversalWindowObservers = mUniversalTargets.mWindows->HasObservers();
   for (auto iter = alerts.begin(); iter < alerts.end(); ++iter) {
+    MOZ_ASSERT(iter);
     RefPtr<nsPerformanceGroup> group = *iter;
     group->SetHasPendingAlert(false);
 
@@ -1300,19 +1385,24 @@ nsPerformanceStatsService::NotifyJankObservers() {
       group->ObservationTarget()
     };
 
+    bool isJankInInput = group->HasRecentUserInput();
+
     RefPtr<PerformanceAlert> alert;
     for (nsPerformanceObservationTarget* target : targets) {
       if (!target) {
         continue;
       }
       if (!alert) {
+        const uint32_t reason = nsIPerformanceAlert::REASON_SLOWDOWN
+          | (isJankInAnimation ? nsIPerformanceAlert::REASON_JANK_IN_ANIMATION : 0)
+          | (isJankInInput ? nsIPerformanceAlert::REASON_JANK_IN_INPUT : 0);
         // Wait until we are sure we need to allocate before we allocate.
-        alert = new PerformanceAlert(nsIPerformanceAlert::REASON_JANK, group);
+        alert = new PerformanceAlert(reason, group);
       }
       target->NotifyJankObservers(details, alert);
     }
 
-    group->ResetHighest();
+    group->ResetRecent();
   }
 
 }
@@ -1341,6 +1431,32 @@ nsPerformanceStatsService::GetObservableWindow(uint64_t windowId,
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsPerformanceStatsService::GetAnimationJankLevelThreshold(short* result) {
+  *result = mJankLevelVisibilityThreshold;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPerformanceStatsService::SetAnimationJankLevelThreshold(short value) {
+  mJankLevelVisibilityThreshold = value;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPerformanceStatsService::GetUserInputDelayThreshold(uint64_t* result) {
+  *result = mMaxExpectedDurationOfInteractionUS;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPerformanceStatsService::SetUserInputDelayThreshold(uint64_t value) {
+  mMaxExpectedDurationOfInteractionUS = value;
+  return NS_OK;
+}
+
+
+
 nsPerformanceStatsService::UniversalTargets::UniversalTargets()
   : mAddons(new nsPerformanceObservationTarget())
   , mWindows(new nsPerformanceObservationTarget())
@@ -1353,7 +1469,7 @@ nsPerformanceStatsService::UniversalTargets::UniversalTargets()
  */
 
 /*static*/ nsPerformanceGroup*
-nsPerformanceGroup::Make(JSRuntime* rt,
+nsPerformanceGroup::Make(JSContext* cx,
                          nsPerformanceStatsService* service,
                          const nsAString& name,
                          const nsAString& addonId,
@@ -1363,7 +1479,7 @@ nsPerformanceGroup::Make(JSRuntime* rt,
                          GroupScope scope)
 {
   nsString groupId;
-  ::GenerateUniqueGroupId(rt, service->GetNextId(), processId, groupId);
+  ::GenerateUniqueGroupId(cx, service->GetNextId(), processId, groupId);
   return new nsPerformanceGroup(service, name, groupId, addonId, windowId, processId, isSystem, scope);
 }
 
@@ -1380,6 +1496,7 @@ nsPerformanceGroup::nsPerformanceGroup(nsPerformanceStatsService* service,
   , mScope(scope)
   , mHighestJank(0)
   , mHighestCPOW(0)
+  , mHasRecentUserInput(false)
   , mHasPendingAlert(false)
 {
   mozilla::Unused << mService->mGroups.PutEntry(this);
@@ -1485,9 +1602,19 @@ nsPerformanceGroup::HighestRecentCPOW() {
   return mHighestCPOW;
 }
 
-void
-nsPerformanceGroup::ResetHighest() {
-  mHighestJank = 0;
-  mHighestCPOW = 0;
+bool
+nsPerformanceGroup::HasRecentUserInput() {
+  return mHasRecentUserInput;
 }
 
+void
+nsPerformanceGroup::RecordUserInput() {
+  mHasRecentUserInput = true;
+}
+
+void
+nsPerformanceGroup::ResetRecent() {
+  mHighestJank = 0;
+  mHighestCPOW = 0;
+  mHasRecentUserInput = false;
+}
