@@ -92,15 +92,15 @@ add_task(function* test_bookmarks() {
 
   // test getFolderIdForItem() with bogus item id will throw
   try {
-    let id = bs.getFolderIdForItem(0);
+    bs.getFolderIdForItem(0);
     do_throw("getFolderIdForItem accepted bad input");
-  } catch(ex) {}
+  } catch (ex) {}
 
   // test getFolderIdForItem() with bogus item id will throw
   try {
-    let id = bs.getFolderIdForItem(-1);
+    bs.getFolderIdForItem(-1);
     do_throw("getFolderIdForItem accepted bad input");
-  } catch(ex) {}
+  } catch (ex) {}
 
   // test root parentage
   do_check_eq(bs.getFolderIdForItem(bs.bookmarksMenuFolder), bs.placesRoot);
@@ -185,9 +185,9 @@ add_task(function* test_bookmarks() {
 
   // get item title bad input
   try {
-    let title = bs.getItemTitle(-3);
+    bs.getItemTitle(-3);
     do_throw("getItemTitle accepted bad input");
-  } catch(ex) {}
+  } catch (ex) {}
 
   // get the folder that the bookmark is in
   let folderId = bs.getFolderIdForItem(newId);
@@ -350,7 +350,7 @@ add_task(function* test_bookmarks() {
     rootNode.containerOpen = true;
     do_check_eq(rootNode.childCount, 3);
     rootNode.containerOpen = false;
-  } catch(ex) {
+  } catch (ex) {
     do_throw("test removeFolderChildren() - querying for children failed: " + ex);
   }
   // 3) remove all children
@@ -362,7 +362,7 @@ add_task(function* test_bookmarks() {
     rootNode.containerOpen = true;
     do_check_eq(rootNode.childCount, 0);
     rootNode.containerOpen = false;
-  } catch(ex) {
+  } catch (ex) {
     do_throw("removeFolderChildren(): " + ex);
   }
 
@@ -370,8 +370,8 @@ add_task(function* test_bookmarks() {
 
   // test bookmark id in query output
   try {
-    let options = hs.getNewQueryOptions();
-    let query = hs.getNewQuery();
+    options = hs.getNewQueryOptions();
+    query = hs.getNewQuery();
     query.setFolders([testRoot], 1);
     let result = hs.executeQuery(query, options);
     let rootNode = result.root;
@@ -393,7 +393,7 @@ add_task(function* test_bookmarks() {
     }
     rootNode.containerOpen = false;
   }
-  catch(ex) {
+  catch (ex) {
     do_throw("bookmarks query: " + ex);
   }
 
@@ -409,8 +409,8 @@ add_task(function* test_bookmarks() {
     bs.insertBookmark(testFolder, mURI, bs.DEFAULT_INDEX, "title 2");
 
     // query
-    let options = hs.getNewQueryOptions();
-    let query = hs.getNewQuery();
+    options = hs.getNewQueryOptions();
+    query = hs.getNewQuery();
     query.setFolders([testFolder], 1);
     let result = hs.executeQuery(query, options);
     let rootNode = result.root;
@@ -421,7 +421,7 @@ add_task(function* test_bookmarks() {
     do_check_eq(rootNode.getChild(1).title, "title 2");
     rootNode.containerOpen = false;
   }
-  catch(ex) {
+  catch (ex) {
     do_throw("bookmarks query: " + ex);
   }
 
@@ -466,7 +466,7 @@ add_task(function* test_bookmarks() {
   try {
     bs.getBookmarkURI(testRoot);
     do_throw("getBookmarkURI() should throw for non-bookmark items!");
-  } catch(ex) {}
+  } catch (ex) {}
 
   // test getItemIndex
   let newId12 = bs.insertBookmark(testRoot, uri("http://foo11.com/"), 1, "");
@@ -499,10 +499,10 @@ add_task(function* test_bookmarks() {
 
   // test search on bookmark title ZZZXXXYYY
   try {
-    let options = hs.getNewQueryOptions();
+    options = hs.getNewQueryOptions();
     options.excludeQueries = 1;
     options.queryType = Ci.nsINavHistoryQueryOptions.QUERY_TYPE_BOOKMARKS;
-    let query = hs.getNewQuery();
+    query = hs.getNewQuery();
     query.searchTerms = "ZZZXXXYYY";
     let result = hs.executeQuery(query, options);
     let rootNode = result.root;
@@ -514,17 +514,17 @@ add_task(function* test_bookmarks() {
     do_check_true(node.itemId > 0);
     rootNode.containerOpen = false;
   }
-  catch(ex) {
+  catch (ex) {
     do_throw("bookmarks query: " + ex);
   }
 
   // test dateAdded and lastModified properties
   // for a search query
   try {
-    let options = hs.getNewQueryOptions();
+    options = hs.getNewQueryOptions();
     options.excludeQueries = 1;
     options.queryType = Ci.nsINavHistoryQueryOptions.QUERY_TYPE_BOOKMARKS;
-    let query = hs.getNewQuery();
+    query = hs.getNewQuery();
     query.searchTerms = "ZZZXXXYYY";
     let result = hs.executeQuery(query, options);
     let rootNode = result.root;
@@ -541,15 +541,15 @@ add_task(function* test_bookmarks() {
 
     rootNode.containerOpen = false;
   }
-  catch(ex) {
+  catch (ex) {
     do_throw("bookmarks query: " + ex);
   }
 
   // test dateAdded and lastModified properties
   // for a folder query
   try {
-    let options = hs.getNewQueryOptions();
-    let query = hs.getNewQuery();
+    options = hs.getNewQueryOptions();
+    query = hs.getNewQuery();
     query.setFolders([testRoot], 1);
     let result = hs.executeQuery(query, options);
     let rootNode = result.root;
@@ -570,7 +570,7 @@ add_task(function* test_bookmarks() {
     }
     rootNode.containerOpen = false;
   }
-  catch(ex) {
+  catch (ex) {
     do_throw("bookmarks query: " + ex);
   }
 
@@ -711,7 +711,7 @@ function getChildCount(aFolderId) {
     rootNode.containerOpen = true;
     cc = rootNode.childCount;
     rootNode.containerOpen = false;
-  } catch(ex) {
+  } catch (ex) {
     do_throw("getChildCount failed: " + ex);
   }
   return cc;
