@@ -123,7 +123,7 @@ function sanityCheckTransactionHistory() {
     try {
       f();
       do_throw("PT.entry should throw for invalid input");
-    } catch(ex) {}
+    } catch (ex) {}
   };
   check_entry_throws( () => PT.entry(-1) );
   check_entry_throws( () => PT.entry({}) );
@@ -327,13 +327,13 @@ function* ensureNonExistent(...aGuids) {
 }
 
 add_task(function* test_recycled_transactions() {
-  function ensureTransactThrowsFor(aTransaction) {
+  function* ensureTransactThrowsFor(aTransaction) {
     let [txns, undoPosition] = getTransactionsHistoryState();
     try {
       yield aTransaction.transact();
       do_throw("Shouldn't be able to use the same transaction twice");
     }
-    catch(ex) { }
+    catch (ex) { }
     ensureUndoState(txns, undoPosition);
   }
 
@@ -356,7 +356,7 @@ add_task(function* test_recycled_transactions() {
       yield txn_a.transact();
       do_throw("Shouldn't be able to use the same transaction twice");
     }
-    catch(ex) { }
+    catch (ex) { }
     ensureUndoState();
     yield txn_b.transact();
   });
@@ -451,22 +451,22 @@ add_task(function* test_merge_create_folder_and_item() {
                 , title: "Test Bookmark"
                 , index: bmStartIndex };
 
-  let { folderTxn, bkmTxn } = yield PT.batch(function* () {
+  let [folderTxnResult, bkmTxnResult] = yield PT.batch(function* () {
     let folderTxn = PT.NewFolder(folder_info);
     folder_info.guid = bm_info.parentGuid = yield folderTxn.transact();
     let bkmTxn = PT.NewBookmark(bm_info);
     bm_info.guid = yield bkmTxn.transact();
-    return { folderTxn, bkmTxn };
+    return [folderTxn, bkmTxn];
   });
 
   let ensureDo = function* () {
-    ensureUndoState([[bkmTxn, folderTxn]], 0);
+    ensureUndoState([[bkmTxnResult, folderTxnResult]], 0);
     yield ensureItemsAdded(folder_info, bm_info);
     observer.reset();
   };
 
   let ensureUndo = () => {
-    ensureUndoState([[bkmTxn, folderTxn]], 1);
+    ensureUndoState([[bkmTxnResult, folderTxnResult]], 1);
     ensureItemsRemoved(folder_info, bm_info);
     observer.reset();
   };
@@ -491,7 +491,7 @@ add_task(function* test_move_items_to_folder() {
                    , title: "Bookmark B" };
 
   // Test moving items within the same folder.
-  let [folder_a_txn, bkm_a_txn, bkm_b_txn] = yield PT.batch(function* () {
+  let [folder_a_txn_result, bkm_a_txn_result, bkm_b_txn_result] = yield PT.batch(function* () {
     let folder_a_txn = PT.NewFolder(folder_a_info);
 
     folder_a_info.guid = bkm_a_info.parentGuid = bkm_b_info.parentGuid =
@@ -503,14 +503,14 @@ add_task(function* test_move_items_to_folder() {
     return [folder_a_txn, bkm_a_txn, bkm_b_txn];
   });
 
-  ensureUndoState([[bkm_b_txn, bkm_a_txn, folder_a_txn]], 0);
+  ensureUndoState([[bkm_b_txn_result, bkm_a_txn_result, folder_a_txn_result]], 0);
 
   let moveTxn = PT.Move({ guid:          bkm_a_info.guid
                         , newParentGuid: folder_a_info.guid });
   yield moveTxn.transact();
 
   let ensureDo = () => {
-    ensureUndoState([[moveTxn], [bkm_b_txn, bkm_a_txn, folder_a_txn]], 0);
+    ensureUndoState([[moveTxn], [bkm_b_txn_result, bkm_a_txn_result, folder_a_txn_result]], 0);
     ensureItemsMoved({ guid:          bkm_a_info.guid
                      , oldParentGuid: folder_a_info.guid
                      , newParentGuid: folder_a_info.guid
@@ -519,7 +519,7 @@ add_task(function* test_move_items_to_folder() {
     observer.reset();
   };
   let ensureUndo = () => {
-    ensureUndoState([[moveTxn], [bkm_b_txn, bkm_a_txn, folder_a_txn]], 1);
+    ensureUndoState([[moveTxn], [bkm_b_txn_result, bkm_a_txn_result, folder_a_txn_result]], 1);
     ensureItemsMoved({ guid:          bkm_a_info.guid
                      , oldParentGuid: folder_a_info.guid
                      , newParentGuid: folder_a_info.guid
@@ -537,14 +537,14 @@ add_task(function* test_move_items_to_folder() {
   ensureUndo();
 
   yield PT.clearTransactionsHistory(false, true);
-  ensureUndoState([[bkm_b_txn, bkm_a_txn, folder_a_txn]], 0);
+  ensureUndoState([[bkm_b_txn_result, bkm_a_txn_result, folder_a_txn_result]], 0);
 
   // Test moving items between folders.
   let folder_b_info = createTestFolderInfo("Folder B");
   let folder_b_txn = PT.NewFolder(folder_b_info);
   folder_b_info.guid = yield folder_b_txn.transact();
   ensureUndoState([ [folder_b_txn]
-                  , [bkm_b_txn, bkm_a_txn, folder_a_txn] ], 0);
+                  , [bkm_b_txn_result, bkm_a_txn_result, folder_a_txn_result] ], 0);
 
   moveTxn = PT.Move({ guid:          bkm_a_info.guid
                     , newParentGuid: folder_b_info.guid
@@ -554,7 +554,7 @@ add_task(function* test_move_items_to_folder() {
   ensureDo = () => {
     ensureUndoState([ [moveTxn]
                     , [folder_b_txn]
-                    , [bkm_b_txn, bkm_a_txn, folder_a_txn] ], 0);
+                    , [bkm_b_txn_result, bkm_a_txn_result, folder_a_txn_result] ], 0);
     ensureItemsMoved({ guid:          bkm_a_info.guid
                      , oldParentGuid: folder_a_info.guid
                      , newParentGuid: folder_b_info.guid
@@ -565,7 +565,7 @@ add_task(function* test_move_items_to_folder() {
   ensureUndo = () => {
     ensureUndoState([ [moveTxn]
                     , [folder_b_txn]
-                    , [bkm_b_txn, bkm_a_txn, folder_a_txn] ], 1);
+                    , [bkm_b_txn_result, bkm_a_txn_result, folder_a_txn_result] ], 1);
     ensureItemsMoved({ guid:          bkm_a_info.guid
                      , oldParentGuid: folder_b_info.guid
                      , newParentGuid: folder_a_info.guid
@@ -588,7 +588,7 @@ add_task(function* test_move_items_to_folder() {
   do_check_eq(observer.itemsRemoved.size, 4);
   ensureUndoState([ [moveTxn]
                   , [folder_b_txn]
-                  , [bkm_b_txn, bkm_a_txn, folder_a_txn] ], 3);
+                  , [bkm_b_txn_result, bkm_a_txn_result, folder_a_txn_result] ], 3);
   yield PT.clearTransactionsHistory();
   ensureUndoState();
 });
@@ -596,8 +596,8 @@ add_task(function* test_move_items_to_folder() {
 add_task(function* test_remove_folder() {
   let folder_level_1_info = createTestFolderInfo("Folder Level 1");
   let folder_level_2_info = { title: "Folder Level 2" };
-  let [folder_level_1_txn,
-       folder_level_2_txn] = yield PT.batch(function* () {
+  let [folder_level_1_txn_result,
+       folder_level_2_txn_result] = yield PT.batch(function* () {
     let folder_level_1_txn  = PT.NewFolder(folder_level_1_info);
     folder_level_1_info.guid = yield folder_level_1_txn.transact();
     folder_level_2_info.parentGuid = folder_level_1_info.guid;
@@ -606,7 +606,7 @@ add_task(function* test_remove_folder() {
     return [folder_level_1_txn, folder_level_2_txn];
   });
 
-  ensureUndoState([[folder_level_2_txn, folder_level_1_txn]]);
+  ensureUndoState([[folder_level_2_txn_result, folder_level_1_txn_result]]);
   yield ensureItemsAdded(folder_level_1_info, folder_level_2_info);
   observer.reset();
 
@@ -614,13 +614,13 @@ add_task(function* test_remove_folder() {
   yield remove_folder_2_txn.transact();
 
   ensureUndoState([ [remove_folder_2_txn]
-                  , [folder_level_2_txn, folder_level_1_txn] ]);
+                  , [folder_level_2_txn_result, folder_level_1_txn_result] ]);
   yield ensureItemsRemoved(folder_level_2_info);
 
   // Undo Remove "Folder Level 2"
   yield PT.undo();
   ensureUndoState([ [remove_folder_2_txn]
-                  , [folder_level_2_txn, folder_level_1_txn] ], 1);
+                  , [folder_level_2_txn_result, folder_level_1_txn_result] ], 1);
   yield ensureItemsAdded(folder_level_2_info);
   ensureTimestampsUpdated(folder_level_2_info.guid, true);
   observer.reset();
@@ -628,14 +628,14 @@ add_task(function* test_remove_folder() {
   // Redo Remove "Folder Level 2"
   yield PT.redo();
   ensureUndoState([ [remove_folder_2_txn]
-                  , [folder_level_2_txn, folder_level_1_txn] ]);
+                  , [folder_level_2_txn_result, folder_level_1_txn_result] ]);
   yield ensureItemsRemoved(folder_level_2_info);
   observer.reset();
 
   // Undo it again
   yield PT.undo();
   ensureUndoState([ [remove_folder_2_txn]
-                  , [folder_level_2_txn, folder_level_1_txn] ], 1);
+                  , [folder_level_2_txn_result, folder_level_1_txn_result] ], 1);
   yield ensureItemsAdded(folder_level_2_info);
   ensureTimestampsUpdated(folder_level_2_info.guid, true);
   observer.reset();
@@ -643,14 +643,14 @@ add_task(function* test_remove_folder() {
   // Undo the creation of both folders
   yield PT.undo();
   ensureUndoState([ [remove_folder_2_txn]
-                  , [folder_level_2_txn, folder_level_1_txn] ], 2);
+                  , [folder_level_2_txn_result, folder_level_1_txn_result] ], 2);
   yield ensureItemsRemoved(folder_level_2_info, folder_level_1_info);
   observer.reset();
 
   // Redo the creation of both folders
   yield PT.redo();
   ensureUndoState([ [remove_folder_2_txn]
-                  , [folder_level_2_txn, folder_level_1_txn] ], 1);
+                  , [folder_level_2_txn_result, folder_level_1_txn_result] ], 1);
   yield ensureItemsAdded(folder_level_1_info, folder_level_2_info);
   ensureTimestampsUpdated(folder_level_1_info.guid, true);
   ensureTimestampsUpdated(folder_level_2_info.guid, true);
@@ -659,20 +659,20 @@ add_task(function* test_remove_folder() {
   // Redo Remove "Folder Level 2"
   yield PT.redo();
   ensureUndoState([ [remove_folder_2_txn]
-                  , [folder_level_2_txn, folder_level_1_txn] ]);
+                  , [folder_level_2_txn_result, folder_level_1_txn_result] ]);
   yield ensureItemsRemoved(folder_level_2_info);
   observer.reset();
 
   // Undo everything one last time
   yield PT.undo();
   ensureUndoState([ [remove_folder_2_txn]
-                  , [folder_level_2_txn, folder_level_1_txn] ], 1);
+                  , [folder_level_2_txn_result, folder_level_1_txn_result] ], 1);
   yield ensureItemsAdded(folder_level_2_info);
   observer.reset();
 
   yield PT.undo();
   ensureUndoState([ [remove_folder_2_txn]
-                  , [folder_level_2_txn, folder_level_1_txn] ], 2);
+                  , [folder_level_2_txn_result, folder_level_1_txn_result] ], 2);
   yield ensureItemsRemoved(folder_level_2_info, folder_level_2_info);
   observer.reset();
 
@@ -681,11 +681,12 @@ add_task(function* test_remove_folder() {
 });
 
 add_task(function* test_add_and_remove_bookmarks_with_additional_info() {
-  const testURI = NetUtil.newURI("http://add.remove.tag")
-      , TAG_1 = "TestTag1", TAG_2 = "TestTag2"
-      , KEYWORD = "test_keyword"
-      , POST_DATA = "post_data"
-      , ANNO = { name: "TestAnno", value: "TestAnnoValue" };
+  const testURI = NetUtil.newURI("http://add.remove.tag");
+  const TAG_1 = "TestTag1";
+  const TAG_2 = "TestTag2";
+  const KEYWORD = "test_keyword";
+  const POST_DATA = "post_data";
+  const ANNO = { name: "TestAnno", value: "TestAnnoValue" };
 
   let folder_info = createTestFolderInfo();
   folder_info.guid = yield PT.NewFolder(folder_info).transact();
@@ -749,9 +750,14 @@ add_task(function* test_add_and_remove_bookmarks_with_additional_info() {
   ensureTags([TAG_1]);
 
   // Check if Remove correctly restores keywords, tags and annotations.
+  // Since both bookmarks share the same uri, they also share the keyword that
+  // is not removed along with one of the bookmarks.
   observer.reset();
   yield PT.redo();
-  ensureItemsChanged(...b2_post_creation_changes);
+  ensureItemsChanged({ guid: b2_info.guid
+                     , isAnnoProperty: true
+                     , property: ANNO.name
+                     , newValue: ANNO.value });
   ensureTags([TAG_1, TAG_2]);
 
   // Test Remove for multiple items.
@@ -761,6 +767,10 @@ add_task(function* test_add_and_remove_bookmarks_with_additional_info() {
   yield PT.Remove(folder_info.guid).transact();
   yield ensureItemsRemoved(b1_info, b2_info, folder_info);
   ensureTags([]);
+  // There is no keyword removal notification cause all bookmarks are removed
+  // before the keyword itself, so there's no one to notify.
+  let entry = yield PlacesUtils.keywords.fetch(KEYWORD);
+  Assert.equal(entry, null, "keyword has been removed");
 
   observer.reset();
   yield PT.undo();
@@ -1021,16 +1031,24 @@ add_task(function* test_edit_keyword() {
   bm_info.guid = yield PT.NewBookmark(bm_info).transact();
 
   observer.reset();
-  yield PT.EditKeyword({ guid: bm_info.guid, keyword: KEYWORD }).transact();
+  yield PT.EditKeyword({ guid: bm_info.guid, keyword: KEYWORD, postData: "postData" }).transact();
   ensureKeywordChange(KEYWORD);
+  let entry = yield PlacesUtils.keywords.fetch(KEYWORD);
+  Assert.equal(entry.url.href, bm_info.url.spec);
+  Assert.equal(entry.postData, "postData");
 
   observer.reset();
   yield PT.undo();
   ensureKeywordChange();
+  entry = yield PlacesUtils.keywords.fetch(KEYWORD);
+  Assert.equal(entry, null);
 
   observer.reset();
   yield PT.redo();
   ensureKeywordChange(KEYWORD);
+  entry = yield PlacesUtils.keywords.fetch(KEYWORD);
+  Assert.equal(entry.url.href, bm_info.url.spec);
+  Assert.equal(entry.postData, "postData");
 
   // Cleanup
   observer.reset();
@@ -1043,6 +1061,68 @@ add_task(function* test_edit_keyword() {
   ensureUndoState();
 });
 
+add_task(function* test_edit_specific_keyword() {
+  let bm_info = { parentGuid: rootGuid
+                , url: NetUtil.newURI("http://test.edit.keyword") };
+  bm_info.guid = yield PT.NewBookmark(bm_info).transact();
+  function ensureKeywordChange(aCurrentKeyword = "", aPreviousKeyword = "") {
+    ensureItemsChanged({ guid: bm_info.guid
+                       , property: "keyword"
+                       , newValue: aCurrentKeyword
+                       });
+  }
+
+  yield PlacesUtils.keywords.insert({ keyword: "kw1", url: bm_info.url.spec, postData: "postData1" });
+  yield PlacesUtils.keywords.insert({ keyword: "kw2", url: bm_info.url.spec, postData: "postData2" });
+  bm_info.guid = yield PT.NewBookmark(bm_info).transact();
+
+  observer.reset();
+  yield PT.EditKeyword({ guid: bm_info.guid, keyword: "keyword", oldKeyword: "kw2" }).transact();
+  ensureKeywordChange("keyword", "kw2");
+  let entry = yield PlacesUtils.keywords.fetch("kw1");
+  Assert.equal(entry.url.href, bm_info.url.spec);
+  Assert.equal(entry.postData, "postData1");
+  entry = yield PlacesUtils.keywords.fetch("keyword");
+  Assert.equal(entry.url.href, bm_info.url.spec);
+  Assert.equal(entry.postData, "postData2");
+  entry = yield PlacesUtils.keywords.fetch("kw2");
+  Assert.equal(entry, null);
+
+  observer.reset();
+  yield PT.undo();
+  ensureKeywordChange("kw2", "keyword");
+  entry = yield PlacesUtils.keywords.fetch("kw1");
+  Assert.equal(entry.url.href, bm_info.url.spec);
+  Assert.equal(entry.postData, "postData1");
+  entry = yield PlacesUtils.keywords.fetch("kw2");
+  Assert.equal(entry.url.href, bm_info.url.spec);
+  Assert.equal(entry.postData, "postData2");
+  entry = yield PlacesUtils.keywords.fetch("keyword");
+  Assert.equal(entry, null);
+
+  observer.reset();
+  yield PT.redo();
+  ensureKeywordChange("keyword", "kw2");
+  entry = yield PlacesUtils.keywords.fetch("kw1");
+  Assert.equal(entry.url.href, bm_info.url.spec);
+  Assert.equal(entry.postData, "postData1");
+  entry = yield PlacesUtils.keywords.fetch("keyword");
+  Assert.equal(entry.url.href, bm_info.url.spec);
+  Assert.equal(entry.postData, "postData2");
+  entry = yield PlacesUtils.keywords.fetch("kw2");
+  Assert.equal(entry, null);
+
+  // Cleanup
+  observer.reset();
+  yield PT.undo();
+  ensureKeywordChange("kw2");
+  yield PT.undo();
+  ensureItemsRemoved(bm_info);
+
+  yield PT.clearTransactionsHistory();
+  ensureUndoState();
+});
+
 add_task(function* test_tag_uri() {
   // This also tests passing uri specs.
   let bm_info_a = { url: "http://bookmarked.uri"
@@ -1134,7 +1214,7 @@ add_task(function* test_untag_uri() {
   });
 
   function* doTest(aInfo) {
-    let urls, tagsToRemove;
+    let urls, tagsRemoved;
     if (aInfo instanceof Ci.nsIURI) {
       urls = [aInfo];
       tagsRemoved = [];
@@ -1298,9 +1378,9 @@ add_task(function* test_sort_folder_by_name() {
   let folder_info = createTestFolderInfo();
 
   let url = NetUtil.newURI("http://sort.by.name/");
-  let preSep =  ["3","2","1"].map(i => ({ title: i, url }));
+  let preSep =  ["3", "2", "1"].map(i => ({ title: i, url }));
   let sep = {};
-  let postSep = ["c","b","a"].map(l => ({ title: l, url }));
+  let postSep = ["c", "b", "a"].map(l => ({ title: l, url }));
   let originalOrder = [...preSep, sep, ...postSep];
   let sortedOrder = [...preSep.slice(0).reverse(),
                      sep,
@@ -1421,9 +1501,9 @@ add_task(function* test_copy() {
   }
 
   // Test duplicating leafs (bookmark, separator, empty folder)
-  let bmTxn = PT.NewBookmark({ url: new URL("http://test.item.duplicate")
-                             , parentGuid: rootGuid
-                             , annos: [{ name: "Anno", value: "AnnoValue"}] });
+  PT.NewBookmark({ url: new URL("http://test.item.duplicate")
+                 , parentGuid: rootGuid
+                 , annos: [{ name: "Anno", value: "AnnoValue"}] });
   let sepTxn = PT.NewSeparator({ parentGuid: rootGuid, index: 1 });
   let livemarkTxn = PT.NewLivemark(
     { feedUrl: new URL("http://test.feed.uri")
@@ -1584,7 +1664,7 @@ add_task(function* test_remove_multiple() {
     let nestedFolderGuid =
       yield PT.NewFolder({ title: "Nested Test Folder"
                          , parentGuid: folderGuid }).transact();
-    let nestedSepGuid = yield PT.NewSeparator(nestedFolderGuid).transact();
+    yield PT.NewSeparator(nestedFolderGuid).transact();
 
     guids.push(folderGuid);
 
