@@ -33,6 +33,12 @@
 #include "nsExceptionHandler.h"
 #endif
 
+#if defined(XP_WIN)
+#include <windows.h>
+#else
+#include <unistd.h>
+#endif
+
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/DebugOnly.h"
@@ -40,7 +46,7 @@
 #include "mozilla/Preferences.h"
 #include "mozilla/Services.h"
 #include "mozilla/UniquePtr.h"
-#include "mozilla/unused.h"
+#include "mozilla/Unused.h"
 #include "mozilla/Telemetry.h"
 
 // Normally, the number of milliseconds that AsyncShutdown waits until
@@ -53,9 +59,6 @@
 // forcefully.
 #define ADDITIONAL_WAIT_BEFORE_CRASH_MS 3000
 
-// One second, in ticks.
-#define TICK_DURATION 1000
-
 namespace mozilla {
 
 namespace {
@@ -141,7 +144,11 @@ RunWatchdog(void* arg)
     // we have lost at most one second, which is much
     // more reasonable.
     //
-    PR_Sleep(TICK_DURATION);
+#if defined(XP_WIN)
+    Sleep(1000 /* ms */);
+#else
+    usleep(1000000 /* usec */);
+#endif
 
     if (gHeartbeat++ < timeToLive) {
       continue;
@@ -166,7 +173,7 @@ RunWatchdog(void* arg)
 class PR_CloseDelete
 {
 public:
-  MOZ_CONSTEXPR PR_CloseDelete() {}
+  constexpr PR_CloseDelete() {}
 
   PR_CloseDelete(const PR_CloseDelete& aOther)
   {}
@@ -301,7 +308,7 @@ struct ShutdownStep
   char const* const mTopic;
   int mTicks;
 
-  MOZ_CONSTEXPR explicit ShutdownStep(const char *const topic)
+  constexpr explicit ShutdownStep(const char *const topic)
     : mTopic(topic)
     , mTicks(-1)
   {}
@@ -337,9 +344,7 @@ nsTerminator::SelfInit()
 
   for (size_t i = 0; i < ArrayLength(sShutdownSteps); ++i) {
     DebugOnly<nsresult> rv = os->AddObserver(this, sShutdownSteps[i].mTopic, false);
-#if defined(DEBUG)
-    NS_WARN_IF(NS_FAILED(rv));
-#endif // defined(DEBUG)
+    NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "AddObserver failed");
   }
 
   return NS_OK;
@@ -351,7 +356,12 @@ nsTerminator::Start()
 {
   MOZ_ASSERT(!mInitialized);
   StartWatchdog();
+#if !defined(DEBUG)
+  // Only allow nsTerminator to write on non-debug builds so we don't get leak warnings on
+  // shutdown for intentional leaks (see bug 1242084). This will be enabled again by bug
+  // 1255484 when 1255478 lands.
   StartWriter();
+#endif // !defined(DEBUG)
   mInitialized = true;
 }
 
@@ -378,7 +388,8 @@ nsTerminator::StartWatchdog()
   }
 
   UniquePtr<Options> options(new Options());
-  options->crashAfterTicks = crashAfterMS / TICK_DURATION;
+  const PRIntervalTime ticksDuration = PR_MillisecondsToInterval(1000);
+  options->crashAfterTicks = crashAfterMS / ticksDuration;
 
   DebugOnly<PRThread*> watchdogThread = CreateSystemThread(RunWatchdog,
                                                 options.release());
@@ -391,7 +402,6 @@ nsTerminator::StartWatchdog()
 void
 nsTerminator::StartWriter()
 {
-
   if (!Telemetry::CanRecordExtended()) {
     return;
   }
@@ -440,7 +450,12 @@ nsTerminator::Observe(nsISupports *, const char *aTopic, const char16_t *)
   }
 
   UpdateHeartbeat(aTopic);
+#if !defined(DEBUG)
+  // Only allow nsTerminator to write on non-debug builds so we don't get leak warnings on
+  // shutdown for intentional leaks (see bug 1242084). This will be enabled again by bug
+  // 1255484 when 1255478 lands.
   UpdateTelemetry();
+#endif // !defined(DEBUG)
   UpdateCrashReport(aTopic);
 
   // Perform a little cleanup
