@@ -15,5 +15,10 @@ function* runTests() {
       next();
     },
   });
-  yield true;
+  yield new Promise(resolve => {
+    bgAddPageThumbObserver(url).catch(function(err) {
+      ok(true, `page-thumbnail error thrown for ${url}`);
+      resolve();
+    });
+  });
 }
