@@ -7,21 +7,21 @@
 #import <AppKit/AppKit.h>
 #include "imgIRequest.h"
 #include "imgIContainer.h"
+#include "nsICancelable.h"
 #include "nsIStringBundle.h"
 #include "nsNetUtil.h"
-#include "imgLoader.h"
 #import "nsCocoaUtils.h"
+#include "nsComponentManagerUtils.h"
 #include "nsContentUtils.h"
 #include "nsObjCExceptions.h"
 #include "nsString.h"
 #include "nsCOMPtr.h"
 #include "nsIObserver.h"
-#include "nsIContentPolicy.h"
-#include "nsAlertsUtils.h"
-#include "imgRequestProxy.h"
 
 using namespace mozilla;
 
+#define MAX_NOTIFICATION_NAME_LEN 5000
+
 #if !defined(MAC_OS_X_VERSION_10_8) || (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_8)
 @protocol NSUserNotificationCenterDelegate
 @end
@@ -148,23 +148,24 @@ enum {
   OSXNotificationActionSettings = 1,
 };
 
-class OSXNotificationInfo {
+class OSXNotificationInfo final : public nsISupports {
 private:
-  ~OSXNotificationInfo();
+  virtual ~OSXNotificationInfo();
 
 public:
-  NS_INLINE_DECL_REFCOUNTING(OSXNotificationInfo)
+  NS_DECL_ISUPPORTS
   OSXNotificationInfo(NSString *name, nsIObserver *observer,
                       const nsAString & alertCookie);
 
   NSString *mName;
   nsCOMPtr<nsIObserver> mObserver;
   nsString mCookie;
-  RefPtr<imgRequestProxy> mIconRequest;
+  RefPtr<nsICancelable> mIconRequest;
   id<FakeNSUserNotification> mPendingNotifiction;
-  nsCOMPtr<nsITimer> mIconTimeoutTimer;
 };
 
+NS_IMPL_ISUPPORTS0(OSXNotificationInfo)
+
 OSXNotificationInfo::OSXNotificationInfo(NSString *name, nsIObserver *observer,
                                          const nsAString & alertCookie)
 {
@@ -218,7 +219,8 @@ OSXNotificationCenter::~OSXNotificationCenter()
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
-NS_IMPL_ISUPPORTS(OSXNotificationCenter, nsIAlertsService, imgINotificationObserver, nsITimerCallback)
+NS_IMPL_ISUPPORTS(OSXNotificationCenter, nsIAlertsService, nsIAlertsIconData,
+                  nsIAlertNotificationImageListener)
 
 nsresult OSXNotificationCenter::Init()
 {
@@ -239,16 +241,57 @@ OSXNotificationCenter::ShowAlertNotification(const nsAString & aImageUrl, const
                                              const nsAString & aLang,
                                              const nsAString & aData,
                                              nsIPrincipal * aPrincipal,
-                                             bool aInPrivateBrowsing)
+                                             bool aInPrivateBrowsing,
+                                             bool aRequireInteraction)
+{
+  nsCOMPtr<nsIAlertNotification> alert =
+    do_CreateInstance(ALERT_NOTIFICATION_CONTRACTID);
+  NS_ENSURE_TRUE(alert, NS_ERROR_FAILURE);
+  nsresult rv = alert->Init(aAlertName, aImageUrl, aAlertTitle,
+                            aAlertText, aAlertTextClickable,
+                            aAlertCookie, aBidi, aLang, aData,
+                            aPrincipal, aInPrivateBrowsing,
+                            aRequireInteraction);
+  NS_ENSURE_SUCCESS(rv, rv);
+  return ShowAlert(alert, aAlertListener);
+}
+
+NS_IMETHODIMP
+OSXNotificationCenter::ShowPersistentNotification(const nsAString& aPersistentData,
+                                                  nsIAlertNotification* aAlert,
+                                                  nsIObserver* aAlertListener)
+{
+  return ShowAlert(aAlert, aAlertListener);
+}
+
+NS_IMETHODIMP
+OSXNotificationCenter::ShowAlert(nsIAlertNotification* aAlert,
+                                 nsIObserver* aAlertListener)
+{
+  return ShowAlertWithIconData(aAlert, aAlertListener, 0, nullptr);
+}
+
+NS_IMETHODIMP
+OSXNotificationCenter::ShowAlertWithIconData(nsIAlertNotification* aAlert,
+                                             nsIObserver* aAlertListener,
+                                             uint32_t aIconSize,
+                                             const uint8_t* aIconData)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
 
+  NS_ENSURE_ARG(aAlert);
+
   Class unClass = NSClassFromString(@"NSUserNotification");
   id<FakeNSUserNotification> notification = [[unClass alloc] init];
-  notification.title = nsCocoaUtils::ToNSString(aAlertTitle);
+
+  nsAutoString title;
+  nsresult rv = aAlert->GetTitle(title);
+  NS_ENSURE_SUCCESS(rv, rv);
+  notification.title = nsCocoaUtils::ToNSString(title);
 
   nsAutoString hostPort;
-  nsAlertsUtils::GetSourceHostPort(aPrincipal, hostPort);
+  rv = aAlert->GetSource(hostPort);
+  NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsIStringBundle> bundle;
   nsCOMPtr<nsIStringBundleService> sbs = do_GetService(NS_STRINGBUNDLE_CONTRACTID);
   sbs->CreateBundle("chrome://alerts/locale/alert.properties", getter_AddRefs(bundle));
@@ -256,104 +299,118 @@ OSXNotificationCenter::ShowAlertNotification(const nsAString & aImageUrl, const
   if (!hostPort.IsEmpty() && bundle) {
     const char16_t* formatStrings[] = { hostPort.get() };
     nsXPIDLString notificationSource;
-    bundle->FormatStringFromName(MOZ_UTF16("source.label"),
+    bundle->FormatStringFromName(u"source.label",
                                  formatStrings,
                                  ArrayLength(formatStrings),
                                  getter_Copies(notificationSource));
     notification.subtitle = nsCocoaUtils::ToNSString(notificationSource);
   }
 
-  notification.informativeText = nsCocoaUtils::ToNSString(aAlertText);
+  nsAutoString text;
+  rv = aAlert->GetText(text);
+  NS_ENSURE_SUCCESS(rv, rv);
+  notification.informativeText = nsCocoaUtils::ToNSString(text);
+
   notification.soundName = NSUserNotificationDefaultSoundName;
   notification.hasActionButton = NO;
 
   // If this is not an application/extension alert, show additional actions dealing with permissions.
-  if (nsAlertsUtils::IsActionablePrincipal(aPrincipal)) {
-    if (bundle) {
-      nsXPIDLString closeButtonTitle, actionButtonTitle, disableButtonTitle, settingsButtonTitle;
-      bundle->GetStringFromName(MOZ_UTF16("closeButton.title"),
-                                getter_Copies(closeButtonTitle));
-      bundle->GetStringFromName(MOZ_UTF16("actionButton.label"),
-                                getter_Copies(actionButtonTitle));
-      if (!hostPort.IsEmpty()) {
-        const char16_t* formatStrings[] = { hostPort.get() };
-        bundle->FormatStringFromName(MOZ_UTF16("webActions.disableForOrigin.label"),
-                                     formatStrings,
-                                     ArrayLength(formatStrings),
-                                     getter_Copies(disableButtonTitle));
-      }
-      bundle->GetStringFromName(MOZ_UTF16("webActions.settings.label"),
-                                getter_Copies(settingsButtonTitle));
+  bool isActionable;
+  if (bundle && NS_SUCCEEDED(aAlert->GetActionable(&isActionable)) && isActionable) {
+    nsXPIDLString closeButtonTitle, actionButtonTitle, disableButtonTitle, settingsButtonTitle;
+    bundle->GetStringFromName(u"closeButton.title",
+                              getter_Copies(closeButtonTitle));
+    bundle->GetStringFromName(u"actionButton.label",
+                              getter_Copies(actionButtonTitle));
+    if (!hostPort.IsEmpty()) {
+      const char16_t* formatStrings[] = { hostPort.get() };
+      bundle->FormatStringFromName(u"webActions.disableForOrigin.label",
+                                   formatStrings,
+                                   ArrayLength(formatStrings),
+                                   getter_Copies(disableButtonTitle));
+    }
+    bundle->GetStringFromName(u"webActions.settings.label",
+                              getter_Copies(settingsButtonTitle));
 
-      notification.otherButtonTitle = nsCocoaUtils::ToNSString(closeButtonTitle);
+    notification.otherButtonTitle = nsCocoaUtils::ToNSString(closeButtonTitle);
 
-      // OS X 10.8 only shows action buttons if the "Alerts" style is set in
-      // Notification Center preferences, and doesn't support the alternate
-      // action menu.
-      if ([notification respondsToSelector:@selector(set_showsButtons:)] &&
-          [notification respondsToSelector:@selector(set_alwaysShowAlternateActionMenu:)] &&
-          [notification respondsToSelector:@selector(set_alternateActionButtonTitles:)]) {
+    // OS X 10.8 only shows action buttons if the "Alerts" style is set in
+    // Notification Center preferences, and doesn't support the alternate
+    // action menu.
+    if ([notification respondsToSelector:@selector(set_showsButtons:)] &&
+        [notification respondsToSelector:@selector(set_alwaysShowAlternateActionMenu:)] &&
+        [notification respondsToSelector:@selector(set_alternateActionButtonTitles:)]) {
 
-        notification.hasActionButton = YES;
-        notification.actionButtonTitle = nsCocoaUtils::ToNSString(actionButtonTitle);
+      notification.hasActionButton = YES;
+      notification.actionButtonTitle = nsCocoaUtils::ToNSString(actionButtonTitle);
 
-        [(NSObject*)notification setValue:@(YES) forKey:@"_showsButtons"];
-        [(NSObject*)notification setValue:@(YES) forKey:@"_alwaysShowAlternateActionMenu"];
-        [(NSObject*)notification setValue:@[
-                                            nsCocoaUtils::ToNSString(disableButtonTitle),
-                                            nsCocoaUtils::ToNSString(settingsButtonTitle)
-                                            ]
-                                 forKey:@"_alternateActionButtonTitles"];
-      }
+      [(NSObject*)notification setValue:@(YES) forKey:@"_showsButtons"];
+      [(NSObject*)notification setValue:@(YES) forKey:@"_alwaysShowAlternateActionMenu"];
+      [(NSObject*)notification setValue:@[
+                                          nsCocoaUtils::ToNSString(disableButtonTitle),
+                                          nsCocoaUtils::ToNSString(settingsButtonTitle)
+                                          ]
+                               forKey:@"_alternateActionButtonTitles"];
     }
   }
-  NSString *alertName = nsCocoaUtils::ToNSString(aAlertName);
+  nsAutoString name;
+  rv = aAlert->GetName(name);
+  // Don't let an alert name be more than MAX_NOTIFICATION_NAME_LEN characters.
+  // More than that shouldn't be necessary and userInfo (assigned to below) has
+  // a length limit of 16k on OS X 10.11. Exception thrown if limit exceeded.
+  if (name.Length() > MAX_NOTIFICATION_NAME_LEN) {
+    return NS_ERROR_FAILURE;
+  }
+
+  NS_ENSURE_SUCCESS(rv, rv);
+  NSString *alertName = nsCocoaUtils::ToNSString(name);
   if (!alertName) {
     return NS_ERROR_FAILURE;
   }
   notification.userInfo = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:alertName, nil]
                                                       forKeys:[NSArray arrayWithObjects:@"name", nil]];
 
-  OSXNotificationInfo *osxni = new OSXNotificationInfo(alertName, aAlertListener, aAlertCookie);
+  nsAutoString cookie;
+  rv = aAlert->GetCookie(cookie);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  OSXNotificationInfo *osxni = new OSXNotificationInfo(alertName, aAlertListener, cookie);
+
+  // Show the favicon if supported on this version of OS X.
+  if (aIconSize > 0 &&
+      [notification respondsToSelector:@selector(set_identityImage:)] &&
+      [notification respondsToSelector:@selector(set_identityImageHasBorder:)]) {
+
+    NSData *iconData = [NSData dataWithBytes:aIconData length:aIconSize];
+    NSImage *icon = [[[NSImage alloc] initWithData:iconData] autorelease];
+
+    [(NSObject*)notification setValue:icon forKey:@"_identityImage"];
+    [(NSObject*)notification setValue:@(NO) forKey:@"_identityImageHasBorder"];
+  }
+
+  bool inPrivateBrowsing;
+  rv = aAlert->GetInPrivateBrowsing(&inPrivateBrowsing);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // Show the notification without waiting for an image if there is no icon URL or
   // notification icons are not supported on this version of OS X.
-  if (aImageUrl.IsEmpty() || ![unClass instancesRespondToSelector:@selector(setContentImage:)]) {
+  if (![unClass instancesRespondToSelector:@selector(setContentImage:)]) {
     CloseAlertCocoaString(alertName);
     mActiveAlerts.AppendElement(osxni);
     [GetNotificationCenter() deliverNotification:notification];
     [notification release];
     if (aAlertListener) {
-      aAlertListener->Observe(nullptr, "alertshow", PromiseFlatString(aAlertCookie).get());
+      aAlertListener->Observe(nullptr, "alertshow", cookie.get());
     }
   } else {
     mPendingAlerts.AppendElement(osxni);
     osxni->mPendingNotifiction = notification;
-    RefPtr<imgLoader> il = imgLoader::GetInstance();
-    if (il) {
-      nsCOMPtr<nsIURI> imageUri;
-      NS_NewURI(getter_AddRefs(imageUri), aImageUrl);
-      if (imageUri) {
-        nsresult rv = il->LoadImage(imageUri, nullptr, nullptr,
-                                    mozilla::net::RP_Default,
-                                    aPrincipal, nullptr,
-                                    this, nullptr,
-                                    aInPrivateBrowsing ? nsIRequest::LOAD_ANONYMOUS :
-                                                         nsIRequest::LOAD_NORMAL,
-                                    nullptr, nsIContentPolicy::TYPE_INTERNAL_IMAGE,
-                                    EmptyString(),
-                                    getter_AddRefs(osxni->mIconRequest));
-        if (NS_SUCCEEDED(rv)) {
-          // Set a timer for six seconds. If we don't have an icon by the time this
-          // goes off then we go ahead without an icon.
-          nsCOMPtr<nsITimer> timer = do_CreateInstance(NS_TIMER_CONTRACTID);
-          osxni->mIconTimeoutTimer = timer;
-          timer->InitWithCallback(this, 6000, nsITimer::TYPE_ONE_SHOT);
-          return NS_OK;
-        }
-      }
+    // Wait six seconds for the image to load.
+    rv = aAlert->LoadImage(6000, this, osxni,
+                           getter_AddRefs(osxni->mIconRequest));
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      ShowPendingNotification(osxni);
     }
-    ShowPendingNotification(osxni);
   }
 
   return NS_OK;
@@ -399,6 +456,10 @@ OSXNotificationCenter::CloseAlertCocoaString(NSString *aAlertName)
       if (osxni->mObserver) {
         osxni->mObserver->Observe(nullptr, "alertfinished", osxni->mCookie.get());
       }
+      if (osxni->mIconRequest) {
+        osxni->mIconRequest->Cancel(NS_BINDING_ABORTED);
+        osxni->mIconRequest = nullptr;
+      }
       mActiveAlerts.RemoveElementAt(i);
       break;
     }
@@ -454,11 +515,6 @@ OSXNotificationCenter::ShowPendingNotification(OSXNotificationInfo *osxni)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
 
-  if (osxni->mIconTimeoutTimer) {
-    osxni->mIconTimeoutTimer->Cancel();
-    osxni->mIconTimeoutTimer = nullptr;
-  }
-
   if (osxni->mIconRequest) {
     osxni->mIconRequest->Cancel(NS_BINDING_ABORTED);
     osxni->mIconRequest = nullptr;
@@ -487,33 +543,14 @@ OSXNotificationCenter::ShowPendingNotification(OSXNotificationInfo *osxni)
 }
 
 NS_IMETHODIMP
-OSXNotificationCenter::Notify(imgIRequest *aRequest, int32_t aType, const nsIntRect* aData)
+OSXNotificationCenter::OnImageMissing(nsISupports* aUserData)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
 
-  if (aType == imgINotificationObserver::LOAD_COMPLETE) {
-    OSXNotificationInfo *osxni = nullptr;
-    for (unsigned int i = 0; i < mPendingAlerts.Length(); i++) {
-      if (aRequest == mPendingAlerts[i]->mIconRequest) {
-        osxni = mPendingAlerts[i];
-        break;
-      }
-    }
-    if (!osxni || !osxni->mPendingNotifiction) {
-      return NS_ERROR_FAILURE;
-    }
-    NSImage *cocoaImage = nil;
-    uint32_t imgStatus = imgIRequest::STATUS_ERROR;
-    nsresult rv = aRequest->GetImageStatus(&imgStatus);
-    if (NS_SUCCEEDED(rv) && !(imgStatus & imgIRequest::STATUS_ERROR)) {
-      nsCOMPtr<imgIContainer> image;
-      rv = aRequest->GetImage(getter_AddRefs(image));
-      if (NS_SUCCEEDED(rv)) {
-        nsCocoaUtils::CreateNSImageFromImageContainer(image, imgIContainer::FRAME_FIRST, &cocoaImage, 1.0f);
-      }
-    }
-    (osxni->mPendingNotifiction).contentImage = cocoaImage;
-    [cocoaImage release];
+  OSXNotificationInfo *osxni = static_cast<OSXNotificationInfo*>(aUserData);
+  if (osxni->mPendingNotifiction) {
+    // If there was an error getting the image, or the request timed out, show
+    // the notification without a content image.
     ShowPendingNotification(osxni);
   }
   return NS_OK;
@@ -522,24 +559,28 @@ OSXNotificationCenter::Notify(imgIRequest *aRequest, int32_t aType, const nsIntR
 }
 
 NS_IMETHODIMP
-OSXNotificationCenter::Notify(nsITimer *timer)
+OSXNotificationCenter::OnImageReady(nsISupports* aUserData,
+                                    imgIRequest* aRequest)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
 
-  if (!timer) {
+  nsCOMPtr<imgIContainer> image;
+  nsresult rv = aRequest->GetImage(getter_AddRefs(image));
+  if (NS_WARN_IF(NS_FAILED(rv) || !image)) {
+    return rv;
+  }
+
+  OSXNotificationInfo *osxni = static_cast<OSXNotificationInfo*>(aUserData);
+  if (!osxni->mPendingNotifiction) {
     return NS_ERROR_FAILURE;
   }
 
-  for (unsigned int i = 0; i < mPendingAlerts.Length(); i++) {
-    OSXNotificationInfo *osxni = mPendingAlerts[i];
-    if (timer == osxni->mIconTimeoutTimer.get()) {
-      osxni->mIconTimeoutTimer = nullptr;
-      if (osxni->mPendingNotifiction) {
-        ShowPendingNotification(osxni);
-        break;
-      }
-    }
-  }
+  NSImage *cocoaImage = nil;
+  nsCocoaUtils::CreateNSImageFromImageContainer(image, imgIContainer::FRAME_FIRST, &cocoaImage, 1.0f);
+  (osxni->mPendingNotifiction).contentImage = cocoaImage;
+  [cocoaImage release];
+  ShowPendingNotification(osxni);
+
   return NS_OK;
 
   NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
