@@ -7,6 +7,7 @@
 #include "nsFilePickerProxy.h"
 #include "nsComponentManagerUtils.h"
 #include "nsIFile.h"
+#include "mozilla/dom/Directory.h"
 #include "mozilla/dom/File.h"
 #include "mozilla/dom/TabChild.h"
 #include "mozilla/dom/ipc/BlobChild.h"
@@ -16,6 +17,7 @@ using namespace mozilla::dom;
 NS_IMPL_ISUPPORTS(nsFilePickerProxy, nsIFilePicker)
 
 nsFilePickerProxy::nsFilePickerProxy()
+  : mSelectedType(0)
 {
 }
 
@@ -24,7 +26,7 @@ nsFilePickerProxy::~nsFilePickerProxy()
 }
 
 NS_IMETHODIMP
-nsFilePickerProxy::Init(nsIDOMWindow* aParent, const nsAString& aTitle,
+nsFilePickerProxy::Init(mozIDOMWindowProxy* aParent, const nsAString& aTitle,
                         int16_t aMode)
 {
   TabChild* tabChild = TabChild::GetFrom(aParent);
@@ -32,10 +34,7 @@ nsFilePickerProxy::Init(nsIDOMWindow* aParent, const nsAString& aTitle,
     return NS_ERROR_FAILURE;
   }
 
-  mParent = do_QueryInterface(aParent);
-  if (!mParent->IsInnerWindow()) {
-    mParent = mParent->GetCurrentInnerWindow();
-  }
+  mParent = nsPIDOMWindowOuter::From(aParent);
 
   mMode = aMode;
 
@@ -102,21 +101,21 @@ nsFilePickerProxy::SetFilterIndex(int32_t aFilterIndex)
 NS_IMETHODIMP
 nsFilePickerProxy::GetFile(nsIFile** aFile)
 {
-  MOZ_ASSERT(false, "GetFile is unimplemented; use GetDomfile");
+  MOZ_ASSERT(false, "GetFile is unimplemented; use GetDomFileOrDirectory");
   return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
 nsFilePickerProxy::GetFileURL(nsIURI** aFileURL)
 {
-  MOZ_ASSERT(false, "GetFileURL is unimplemented; use GetDomfile");
+  MOZ_ASSERT(false, "GetFileURL is unimplemented; use GetDomFileOrDirectory");
   return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
 nsFilePickerProxy::GetFiles(nsISimpleEnumerator** aFiles)
 {
-  MOZ_ASSERT(false, "GetFiles is unimplemented; use GetDomfiles");
+  MOZ_ASSERT(false, "GetFiles is unimplemented; use GetDomFileOrDirectoryEnumerator");
   return NS_ERROR_FAILURE;
 }
 
@@ -138,17 +137,17 @@ nsFilePickerProxy::Open(nsIFilePickerShownCallback* aCallback)
   }
 
   SendOpen(mSelectedType, mAddToRecentDocs, mDefault, mDefaultExtension,
-           mFilters, mFilterNames, displayDirectory);
+           mFilters, mFilterNames, displayDirectory, mOkButtonLabel);
 
   return NS_OK;
 }
 
 bool
-nsFilePickerProxy::Recv__delete__(const MaybeInputFiles& aFiles,
+nsFilePickerProxy::Recv__delete__(const MaybeInputData& aData,
                                   const int16_t& aResult)
 {
-  if (aFiles.type() == MaybeInputFiles::TInputFiles) {
-    const InfallibleTArray<PBlobChild*>& blobs = aFiles.get_InputFiles().blobsChild();
+  if (aData.type() == MaybeInputData::TInputBlobs) {
+    const InfallibleTArray<PBlobChild*>& blobs = aData.get_InputBlobs().blobsChild();
     for (uint32_t i = 0; i < blobs.Length(); ++i) {
       BlobChild* actor = static_cast<BlobChild*>(blobs[i]);
       RefPtr<BlobImpl> blobImpl = actor->GetBlobImpl();
@@ -158,11 +157,28 @@ nsFilePickerProxy::Recv__delete__(const MaybeInputFiles& aFiles,
         return true;
       }
 
-      RefPtr<File> file = File::Create(mParent, blobImpl);
+      nsPIDOMWindowInner* inner =
+        mParent ? mParent->GetCurrentInnerWindow() : nullptr;
+      RefPtr<File> file = File::Create(inner, blobImpl);
       MOZ_ASSERT(file);
 
-      mDomfiles.AppendElement(file);
+      OwningFileOrDirectory* element = mFilesOrDirectories.AppendElement();
+      element->SetAsFile() = file;
     }
+  } else if (aData.type() == MaybeInputData::TInputDirectory) {
+    nsCOMPtr<nsIFile> file;
+    NS_ConvertUTF16toUTF8 path(aData.get_InputDirectory().directoryPath());
+    nsresult rv = NS_NewNativeLocalFile(path, true, getter_AddRefs(file));
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return true;
+    }
+
+    RefPtr<Directory> directory =
+      Directory::Create(mParent->GetCurrentInnerWindow(), file);
+    MOZ_ASSERT(directory);
+
+    OwningFileOrDirectory* element = mFilesOrDirectories.AppendElement();
+    element->SetAsDirectory() = directory;
   }
 
   if (mCallback) {
@@ -174,16 +190,24 @@ nsFilePickerProxy::Recv__delete__(const MaybeInputFiles& aFiles,
 }
 
 NS_IMETHODIMP
-nsFilePickerProxy::GetDomfile(nsISupports** aDomfile)
+nsFilePickerProxy::GetDomFileOrDirectory(nsISupports** aValue)
 {
-  *aDomfile = nullptr;
-  if (mDomfiles.IsEmpty()) {
+  *aValue = nullptr;
+  if (mFilesOrDirectories.IsEmpty()) {
     return NS_OK;
   }
 
-  MOZ_ASSERT(mDomfiles.Length() == 1);
-  nsCOMPtr<nsIDOMBlob> blob = mDomfiles[0].get();
-  blob.forget(aDomfile);
+  MOZ_ASSERT(mFilesOrDirectories.Length() == 1);
+
+  if (mFilesOrDirectories[0].IsFile()) {
+    nsCOMPtr<nsIDOMBlob> blob = mFilesOrDirectories[0].GetAsFile().get();
+    blob.forget(aValue);
+    return NS_OK;
+  }
+
+  MOZ_ASSERT(mFilesOrDirectories[0].IsDirectory());
+  RefPtr<Directory> directory = mFilesOrDirectories[0].GetAsDirectory();
+  directory.forget(aValue);
   return NS_OK;
 }
 
@@ -194,8 +218,9 @@ class SimpleEnumerator final : public nsISimpleEnumerator
 public:
   NS_DECL_ISUPPORTS
 
-  explicit SimpleEnumerator(const nsTArray<RefPtr<File>>& aFiles)
-    : mFiles(aFiles)
+  explicit
+  SimpleEnumerator(const nsTArray<OwningFileOrDirectory>& aFilesOrDirectories)
+    : mFilesOrDirectories(aFilesOrDirectories)
     , mIndex(0)
   {}
 
@@ -203,17 +228,26 @@ public:
   HasMoreElements(bool* aRetvalue) override
   {
     MOZ_ASSERT(aRetvalue);
-    *aRetvalue = mIndex < mFiles.Length();
+    *aRetvalue = mIndex < mFilesOrDirectories.Length();
     return NS_OK;
   }
 
   NS_IMETHOD
-  GetNext(nsISupports** aSupports) override
+  GetNext(nsISupports** aValue) override
   {
-    NS_ENSURE_TRUE(mIndex < mFiles.Length(), NS_ERROR_FAILURE);
+    NS_ENSURE_TRUE(mIndex < mFilesOrDirectories.Length(), NS_ERROR_FAILURE);
 
-    nsCOMPtr<nsIDOMBlob> blob = mFiles[mIndex++].get();
-    blob.forget(aSupports);
+    uint32_t index = mIndex++;
+
+    if (mFilesOrDirectories[index].IsFile()) {
+      nsCOMPtr<nsIDOMBlob> blob = mFilesOrDirectories[index].GetAsFile().get();
+      blob.forget(aValue);
+      return NS_OK;
+    }
+
+    MOZ_ASSERT(mFilesOrDirectories[index].IsDirectory());
+    RefPtr<Directory> directory = mFilesOrDirectories[index].GetAsDirectory();
+    directory.forget(aValue);
     return NS_OK;
   }
 
@@ -221,7 +255,7 @@ private:
   ~SimpleEnumerator()
   {}
 
-  nsTArray<RefPtr<File>> mFiles;
+  nsTArray<mozilla::dom::OwningFileOrDirectory> mFilesOrDirectories;
   uint32_t mIndex;
 };
 
@@ -230,9 +264,10 @@ NS_IMPL_ISUPPORTS(SimpleEnumerator, nsISimpleEnumerator)
 } // namespace
 
 NS_IMETHODIMP
-nsFilePickerProxy::GetDomfiles(nsISimpleEnumerator** aDomfiles)
+nsFilePickerProxy::GetDomFileOrDirectoryEnumerator(nsISimpleEnumerator** aDomfiles)
 {
-  RefPtr<SimpleEnumerator> enumerator = new SimpleEnumerator(mDomfiles);
+  RefPtr<SimpleEnumerator> enumerator =
+    new SimpleEnumerator(mFilesOrDirectories);
   enumerator.forget(aDomfiles);
   return NS_OK;
 }
